@@ -4,14 +4,14 @@
 Constants and exceptions for Pelix.
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Constants and exceptions for Pelix.
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -35,7 +35,12 @@ __docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
-ACTIVATOR = "activator"
+# Standard library
+import inspect
+
+# ------------------------------------------------------------------------------
+
+ACTIVATOR = "__pelix_bundle_activator__"
 """
 Name of the module member that will be used as bundle activator.
 It must be an object with the following methods:
@@ -44,6 +49,17 @@ It must be an object with the following methods:
 * stop(BundleContext)
 """
 
+ACTIVATOR_LEGACY = "activator"
+"""
+Deprecated: prefer ACTIVATOR
+
+Name of the module member that will be used as bundle activator.
+It must be an object with the following methods:
+
+* start(BundleContext)
+* stop(BundleContext)
+"""
+
 OBJECTCLASS = "objectClass"
 """
 Property containing the list of specifications (strings) provided by a service
@@ -80,6 +96,23 @@ This property is constant during the life of a framework instance.
 
 # ------------------------------------------------------------------------------
 
+def BundleActivator(clazz):
+    """
+    Decorator to declare the bundle activator
+
+    Instantiates the decorated class and stores it as a module member.
+
+    :param clazz: The decorated bundle activator class
+    :return: The class itself
+    """
+    # Add the activator instance to the module
+    setattr(inspect.getmodule(clazz), ACTIVATOR, clazz())
+
+    # Return the untouched class
+    return clazz
+
+# ------------------------------------------------------------------------------
+
 class BundleException(Exception):
     """
     The base of all framework exceptions
