@@ -6,14 +6,14 @@ Core module for Pelix.
 Pelix is a Python framework that aims to act as OSGi as much as possible
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@ Pelix is a Python framework that aims to act as OSGi as much as possible
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -51,7 +51,6 @@ from pelix.utilities import SynchronizedClassMethod, is_string
 # Standard library
 import imp
 import importlib
-import inspect
 import logging
 import os
 import pkgutil
@@ -123,6 +122,27 @@ class Bundle(object):
         return "Bundle(ID={0}, Name={1})".format(self.__id, self.__name)
 
 
+    def __get_activator_method(self, method_name):
+        """
+        Retrieves the requested method of the activator, or returns None
+
+        :param method_name: A method name
+        :return: A method, or None
+        """
+        # Get the activator
+        activator = getattr(self.__module, ACTIVATOR, None)
+        if activator is None:
+            # Get the old activator
+            activator = getattr(self.__module, ACTIVATOR_LEGACY, None)
+            if activator is not None:
+                # Old activator found: print a deprecation warning
+                _logger.warning("Bundle %s uses the deprecated '%s' to declare "
+                                "its activator. Use @BundleActivator instead.",
+                                self.__name, ACTIVATOR_LEGACY)
+
+        return getattr(activator, method_name, None)
+
+
     def _fire_bundle_event(self, kind):
         """
         Fires a bundle event of the given kind
@@ -296,9 +316,7 @@ class Bundle(object):
             self._fire_bundle_event(BundleEvent.STARTING)
 
             # Call the activator, if any
-            activator = getattr(self.__module, ACTIVATOR, None)
-            starter = getattr(activator, 'start', None)
-
+            starter = self.__get_activator_method('start')
             if starter is not None:
                 try:
                     # Call the start method
@@ -337,6 +355,7 @@ class Bundle(object):
             # Invalid state
             return
 
+        exception = None
         with self._lock:
             # Store the bundle current state
             previous_state = self._state
@@ -346,10 +365,7 @@ class Bundle(object):
             self._fire_bundle_event(BundleEvent.STOPPING)
 
             # Call the activator, if any
-            activator = getattr(self.__module, ACTIVATOR, None)
-            stopper = getattr(activator, 'stop', None)
-
-            exception = None
+            stopper = self.__get_activator_method('stop')
             if stopper is not None:
                 try:
                     # Call the start method
@@ -795,7 +811,10 @@ class Framework(Bundle):
 
         # Create a simple visitor
         def visitor(fullname, is_package, module_path):
-            # Accept everything in recursive mode, else avoid packages
+            """
+            Package visitor: accepts everything in recursive mode,
+            else avoids packages
+            """
             return recursive or not is_package
 
         # Set up the prefix if needed
@@ -1346,9 +1365,10 @@ class BundleContext(object):
                  calling bundle and matching the filters.
         """
         refs = self.__framework.find_service_references(clazz, ldap_filter)
-        for ref in refs:
-            if ref.get_bundle() is not self.__bundle:
-                refs.remove(ref)
+        if refs:
+            for ref in refs:
+                if ref.get_bundle() is not self.__bundle:
+                    refs.remove(ref)
 
         return refs
 
