@@ -4,14 +4,14 @@
 Definition of Factory and Component context classes
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Definition of Factory and Component context classes
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -35,9 +35,6 @@ __docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
-# Pelix beans
-from pelix.framework import BundleContext
-
 # Pelix utilities
 from pelix.constants import OBJECTCLASS
 from pelix.utilities import is_string
@@ -242,6 +239,12 @@ class FactoryContext(object):
         # Handler ID -> configuration
         self.__handlers = {}
 
+        # Inherited configuration
+        self.__inherited_configuration = {}
+
+        # Instance name -> Instance properties
+        self.__instances = {}
+
 
     def __eq__(self, other):
         """
@@ -266,6 +269,85 @@ class FactoryContext(object):
         return not self.__eq__(other)
 
 
+    def copy(self, inheritance=False):
+        """
+        Returns a deep copy of the current FactoryContext instance
+
+        :param inheritance: If True, current handlers configurations are stored
+                            as inherited ones
+        """
+        # Copy the context
+        new_context = copy.deepcopy(self)
+
+        if inheritance:
+            # Store configuration as inherited one
+            new_context.__inherited_configuration = new_context.__handlers
+            new_context.__handlers = {}
+
+        # Remove instances in any case
+        new_context.__instances.clear()
+        return new_context
+
+
+    def inherit_handlers(self, excluded_handlers):
+        """
+        Merges the inherited configuration with the current ones
+
+        :param excluded_handlers: Excluded handlers
+        """
+        if not excluded_handlers:
+            excluded_handlers = tuple()
+
+        for handler, configuration in self.__inherited_configuration.items():
+            if handler in excluded_handlers:
+                # Excluded handler
+                continue
+
+            elif handler not in self.__handlers:
+                # Fully inherited configuration
+                self.__handlers[handler] = configuration
+
+            # Merge configuration...
+            elif isinstance(configuration, dict):
+                # Dictionary
+                self.__handlers.setdefault(handler, {}).update(configuration)
+
+            elif isinstance(configuration, list):
+                # List
+                handler_conf = self.__handlers.setdefault(handler, [])
+                for item in configuration:
+                    if item not in handler_conf:
+                        handler_conf.append(item)
+
+        # Clear the inherited configuration dictionary
+        self.__inherited_configuration.clear()
+
+
+    def add_instance(self, name, properties):
+        """
+        Stores the description of a component instance. The given properties
+        are stored as is.
+
+        :param name: Instance name
+        :param properties: Instance properties
+        :raise NameError: Already known instance name
+        """
+        if name in self.__instances:
+            raise NameError(name)
+
+        # Store properties "as-is"
+        self.__instances[name] = properties
+
+
+    def get_instances(self):
+        """
+        Returns the dictionary of instances to start: name -> properties
+
+        :return: A dictionary: instance name -> instance properties
+        """
+        return copy.deepcopy(self.__instances)
+
+
     def get_handlers_ids(self):
         """
         Retrieves the IDs of the handlers to instantiate for this component
@@ -273,13 +355,6 @@ class FactoryContext(object):
         return list(self.__handlers.keys())
 
 
-    def copy(self):
-        """
-        Returns a deep copy of the current FactoryContext instance
-        """
-        return copy.deepcopy(self)
-
-
     def get_handler(self, handler_id, default=None):
         """
         Retrieves the configuration associated to the given handler
@@ -288,6 +363,18 @@ class FactoryContext(object):
         :param default: The default configuration value
         :return: The existing configuration or the given default
         """
+        return self.__handlers.get(handler_id, default)
+
+
+    def set_handler_default(self, handler_id, default=None):
+        """
+        Retrieves the configuration associated to the given handler, creates
+        it the entry with the given value if necessary
+
+        :param handler_id: The ID of the configured handler
+        :param default: The default configuration value to store if none exists
+        :return: The existing configuration or the given default
+        """
         return self.__handlers.setdefault(handler_id, default)
 
 
@@ -305,11 +392,9 @@ class FactoryContext(object):
         """
         Sets up the bundle context associated to this factory context
 
-        :param bundle_context: The factory bundle context
+        :param bundle_context: The factory bundle context (or None to clear it)
         """
-        if self.bundle_context is None:
-            assert isinstance(bundle_context, BundleContext)
-            self.bundle_context = bundle_context
+        self.bundle_context = bundle_context
 
 # ------------------------------------------------------------------------------
 
@@ -367,10 +452,11 @@ class ComponentContext(object):
 
         :param field: Name of the dependency field
         :param event: A component life cycle event
-        :return: The callback associated to the given event
+        :return: A 2-tuple containing the callback associated to the given event
+                 and flag indicating if the callback must be called in valid
+                 state only
         """
-        return self.factory_context.field_callbacks.get(field,
-                                                        {}).get(event, None)
+        return self.factory_context.field_callbacks.get(field, {}).get(event)
 
 
     def get_factory_name(self):
