@@ -4,14 +4,14 @@
 Core iPOPO implementation
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Core iPOPO implementation
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -36,10 +36,10 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 # Pelix
+from pelix.constants import SERVICE_ID, BundleActivator
+from pelix.framework import Bundle, BundleException
 from pelix.internals.events import BundleEvent, ServiceEvent
-from pelix.framework import BundleContext, Bundle
 from pelix.utilities import add_listener, remove_listener, is_string
-import pelix.framework as pelix
 
 # iPOPO constants
 import pelix.ipopo.constants as constants
@@ -63,7 +63,8 @@ _logger = logging.getLogger("ipopo.core")
 # Built-in handlers, automatically installed
 BUILTIN_HANDLERS = ('pelix.ipopo.handlers.properties',
                     'pelix.ipopo.handlers.provides',
-                    'pelix.ipopo.handlers.requires')
+                    'pelix.ipopo.handlers.requires',
+                    'pelix.ipopo.handlers.requiresmap')
 
 # ------------------------------------------------------------------------------
 
@@ -130,7 +131,6 @@ class _IPopoService(object):
     """
     The iPOPO registry and service
     """
-
     def __init__(self, bundle_context):
         """
         Sets up the iPOPO registry
@@ -159,11 +159,15 @@ class _IPopoService(object):
         self.__factories_lock = threading.RLock()
         self.__instances_lock = threading.RLock()
         self.__listeners_lock = threading.RLock()
+        self.__handlers_lock = threading.RLock()
 
         # Handlers factories
         self._handlers_refs = set()
         self._handlers = {}
 
+        # Instances waiting for a handler: Name -> (ComponentContext, instance)
+        self.__waiting_handlers = {}
+
         # Register the service listener
         bundle_context.add_service_listener(self, None,
                                 handlers_const.SERVICE_IPOPO_HANDLER_FACTORY)
@@ -189,18 +193,30 @@ class _IPopoService(object):
 
         :param svc_ref: ServiceReference of the new handler factory
         """
-        # Get the handler ID
-        handler_id = svc_ref.get_property(handlers_const.PROP_HANDLER_ID)
+        with self.__handlers_lock:
+            # Get the handler ID
+            handler_id = svc_ref.get_property(handlers_const.PROP_HANDLER_ID)
 
-        if handler_id in self._handlers:
-            # Duplicated ID
-            _logger.warning("Already registered handler ID: %s",
-                            handler_id)
+            if handler_id in self._handlers:
+                # Duplicated ID
+                _logger.warning("Already registered handler ID: %s",
+                                handler_id)
 
-        else:
-            # Store the service
-            self._handlers_refs.add(svc_ref)
-            self._handlers[handler_id] = self.__context.get_service(svc_ref)
+            else:
+                # Store the service
+                self._handlers_refs.add(svc_ref)
+                self._handlers[handler_id] = self.__context.get_service(svc_ref)
+
+                # Try to instantiate waiting components
+                succeeded = set()
+                for name, (context, instance) \
+                in self.__waiting_handlers.items():
+                    if self.__try_instantiate(context, instance):
+                        succeeded.add(name)
+
+                # Remove instantiated component from the waiting list
+                for name in succeeded:
+                    del self.__waiting_handlers[name]
 
 
     def __remove_handler_factory(self, svc_ref):
@@ -209,21 +225,91 @@ class _IPopoService(object):
 
         :param svc_ref: ServiceReference of the handler factory to remove
         """
-        # Get the handler ID
-        handler_id = svc_ref.get_property(handlers_const.PROP_HANDLER_ID)
+        with self.__handlers_lock:
+            # Get the handler ID
+            handler_id = svc_ref.get_property(handlers_const.PROP_HANDLER_ID)
+
+            # Check if this is the handler we use
+            if svc_ref not in self._handlers_refs:
+                return
 
-        try:
             # Clean up
             self.__context.unget_service(svc_ref)
             self._handlers_refs.remove(svc_ref)
             del self._handlers[handler_id]
 
-        except KeyError:
-            # We weren't using this handler
-            pass
+            # List the components using this handler
+            to_stop = set()
+            for factory_name in self.__factories:
+                _, context = self.__get_factory_with_context(factory_name)
+                if handler_id in context.get_handlers_ids():
+                    to_stop.update(self.__get_stored_instances(factory_name))
+
+            with self.__instances_lock:
+                for stored_instance in to_stop:
+                    # Extract information
+                    context = stored_instance.context
+                    name = context.name
+                    instance = stored_instance.instance
+
+                    # Clean up the stored instance (iPOPO side)
+                    del self.__instances[name]
+                    stored_instance.kill()
+
+                    # Add the component to the waiting queue
+                    self.__waiting_handlers[name] = (context, instance)
+
+            # Try to find a new handler factory
+            new_ref = self.__context.get_service_reference(
+                                handlers_const.SERVICE_IPOPO_HANDLER_FACTORY,
+                                "({0}={1})"\
+                                .format(handlers_const.PROP_HANDLER_ID,
+                                        handler_id))
+            if new_ref is not None:
+                self.__add_handler_factory(new_ref)
 
 
-    def __get_stored_instances_by_factory(self, factory_name):
+    def __get_factory_with_context(self, factory_name):
+        """
+        Retrieves the factory registered with the given and its factory context
+
+        :param factory_name: The name of the factory
+        :return: A (factory, context) tuple
+        :raise TypeError: Unknown factory, or factory not manipulated
+        """
+        factory = self.__factories.get(factory_name)
+        if factory is None:
+            raise TypeError("Unknown factory '{0}'" \
+                            .format(factory_name))
+
+        # Get the factory context
+        factory_context = getattr(factory, constants.IPOPO_FACTORY_CONTEXT,
+                                  None)
+        if factory_context is None:
+            raise TypeError("Factory context missing in '{0}'" \
+                            .format(factory_name))
+
+        return factory, factory_context
+
+
+    def __get_handler_factories(self, handlers_ids):
+        """
+        Returns the list of Handler Factories for the given Handlers IDs.
+        Raises a KeyError exception is a handler factory is missing.
+
+        :param handlers_ids: List of handlers IDs
+        :raise KeyError: A handler is missing
+        """
+        # Look for the required handlers
+        handler_factories = set()
+        for handler_id in handlers_ids:
+            # Not a 'set-comprehension': handler_id must be visible
+            handler_factories.add(self._handlers[handler_id])
+
+        return handler_factories
+
+
+    def __get_stored_instances(self, factory_name):
         """
         Retrieves the list of all stored instances objects corresponding to
         the given factory name
@@ -237,6 +323,63 @@ class _IPopoService(object):
                     if stored_instance.factory_name == factory_name]
 
 
+    def __try_instantiate(self, component_context, instance):
+        """
+        Instantiates a component, if all of its handlers are there. Returns
+        False if a handler is missing.
+
+        :param component_context: A ComponentContext bean
+        :param instance: The component instance
+        :return: True if the component has started,
+                 False if a handler is missing
+        """
+        with self.__instances_lock:
+            # Extract information about the component
+            factory_context = component_context.factory_context
+            handlers_ids = factory_context.get_handlers_ids()
+            name = component_context.name
+            factory_name = factory_context.name
+
+            try:
+                # Get handlers
+                handler_factories = self.__get_handler_factories(handlers_ids)
+
+            except KeyError:
+                # A handler is missing, stop here
+                return False
+
+            # Instantiate the handlers
+            all_handlers = set()
+            for handler_factory in handler_factories:
+                handlers = handler_factory.get_handlers(component_context,
+                                                        instance)
+                if handlers:
+                    all_handlers.update(handlers)
+
+            # Prepare the stored instance
+            stored_instance = StoredInstance(self, component_context, instance,
+                                             all_handlers)
+
+            # Manipulate the properties
+            for handler in all_handlers:
+                handler.manipulate(stored_instance, instance)
+
+            # Store the instance
+            self.__instances[name] = stored_instance
+
+        # Start the manager
+        stored_instance.start()
+
+        # Notify listeners now that every thing is ready to run
+        self._fire_ipopo_event(constants.IPopoEvent.INSTANTIATED,
+                               factory_name, name)
+
+        # Try to validate it
+        stored_instance.update_bindings()
+        stored_instance.check_lifecycle()
+        return True
+
+
     def _autorestart_store_components(self, bundle):
         """
         Stores the components of the given bundle with the auto-restart property
@@ -300,27 +443,27 @@ class _IPopoService(object):
                 del self.__auto_restart[bundle]
 
 
-    def _fire_ipopo_event(self, kind, factory_name, component_name=None):
+    def _fire_ipopo_event(self, kind, factory_name, instance_name=None):
         """
         Triggers an iPOPO event
 
         :param kind: Kind of event
         :param factory_name: Name of the factory associated to the event
-        :param component_name: Name of the component instance associated to the
-                               event
+        :param instance_name: Name of the component instance associated to the
+                              event
         """
         with self.__listeners_lock:
             # Use a copy of the list of listeners
             listeners = self.__listeners[:]
 
-            for listener in listeners:
-                try:
-                    listener.handle_ipopo_event(constants.IPopoEvent(kind,
-                                                             factory_name,
-                                                             component_name))
+        for listener in listeners:
+            try:
+                listener.handle_ipopo_event(constants.IPopoEvent(kind,
+                                                                 factory_name,
+                                                                 instance_name))
 
-                except:
-                    _logger.exception("Error calling an iPOPO event handler")
+            except:
+                _logger.exception("Error calling an iPOPO event handler")
 
 
     def _prepare_instance_properties(self, properties, factory_properties):
@@ -361,16 +504,23 @@ class _IPopoService(object):
         factories = _load_bundle_factories(bundle)
 
         for context, factory_class in factories:
-            # Register each found factory
-            self._register_factory(context.name, factory_class, True)
+            try:
+                # Register each found factory
+                self._register_factory(context.name, factory_class, False)
 
-            instances = getattr(factory_class, constants.IPOPO_INSTANCES, None)
-            if isinstance(instances, dict):
-                for name, properties in instances.items():
+            except ValueError as ex:
+                # Already known factory
+                _logger.error("Cannot register factory '%s' of bundle %d (%s): "
+                              "%s", context.name, bundle.get_bundle_id(),
+                              bundle.get_symbolic_name(), ex)
+
+            else:
+                # Instantiate components
+                for name, properties in context.get_instances().items():
                     self.instantiate(context.name, name, properties)
 
 
-    def _register_factory(self, factory_name, factory, override=True):
+    def _register_factory(self, factory_name, factory, override):
         """
         Registers a component factory
 
@@ -426,12 +576,9 @@ class _IPopoService(object):
 
         with self.__factories_lock:
             # Find out which factories must be removed
-            to_remove = []
-
-            for factory_name in self.__factories:
-                if self.get_factory_bundle(factory_name) is bundle:
-                    # Found a factory provided by the given bundle
-                    to_remove.append(factory_name)
+            to_remove = [factory_name
+                         for factory_name in self.__factories
+                         if self.get_factory_bundle(factory_name) is bundle]
 
             # Remove all of them
             for factory_name in to_remove:
@@ -549,35 +696,14 @@ class _IPopoService(object):
             raise ValueError("Framework is stopping")
 
         with self.__instances_lock:
-            if name in self.__instances:
+            if name in self.__instances or name in self.__waiting_handlers:
                 raise ValueError("'{0}' is an already running instance name" \
                                  .format(name))
 
             with self.__factories_lock:
-                # Can raise a ValueError exception
-                factory = self.__factories.get(factory_name, None)
-                if factory is None:
-                    raise TypeError("Unknown factory '{0}'" \
-                                    .format(factory_name))
-
-                # Get the factory context
-                factory_context = getattr(factory, \
-                                          constants.IPOPO_FACTORY_CONTEXT, None)
-                if factory_context is None:
-                    raise TypeError("Factory context missing in '{0}'" \
-                                    .format(factory_name))
-
-            try:
-                # Look for the required handlers
-                handler_factories = set()
-                for handler_id in factory_context.get_handlers_ids():
-                    # Not a 'set-comprehension': handler_id must be visible
-                    handler_factories.add(self._handlers[handler_id])
-
-            except KeyError:
-                raise TypeError("Missing handler '{0}' for factory '{1}', "
-                                "component '{2}'" \
-                                .format(handler_id, factory_name, name))
+                # Can raise a TypeError exception
+                factory, factory_context = self.__get_factory_with_context(
+                                                                   factory_name)
 
             # Create component instance
             try:
@@ -598,35 +724,10 @@ class _IPopoService(object):
             component_context = ComponentContext(factory_context, name, \
                                                  properties)
 
-            # Instantiate the handlers
-            all_handlers = set()
-            for handler_factory in handler_factories:
-                handlers = handler_factory.get_handlers(component_context,
-                                                        instance)
-                if handlers:
-                    all_handlers.update(handlers)
-
-            # Prepare the stored instance
-            stored_instance = StoredInstance(self, component_context, instance,
-                                             all_handlers)
-
-            # Manipulate the properties
-            for handler in all_handlers:
-                handler.manipulate(stored_instance, instance)
-
-            # Store the instance
-            self.__instances[name] = stored_instance
-
-        # Start the manager
-        stored_instance.start()
-
-        # Notify listeners now that every thing is ready to run
-        self._fire_ipopo_event(constants.IPopoEvent.INSTANTIATED,
-                               factory_name, name)
-
-        # Try to validate it
-        stored_instance.update_bindings()
-        stored_instance.check_lifecycle()
+        # Try to instantiate the component immediately
+        if not self.__try_instantiate(component_context, instance):
+            # A handler is missing, put the component in the queue
+            self.__waiting_handlers[name] = (component_context, instance)
 
         return instance
 
@@ -661,7 +762,8 @@ class _IPopoService(object):
 
     def is_registered_instance(self, name):
         """
-        Tests if the given name is in the instance registry
+        Tests if the given name is in the instance registry or in the waiting
+        queue
 
         :param name: A component name to be tested
         """
@@ -680,14 +782,21 @@ class _IPopoService(object):
             raise ValueError("Name can't be None or empty")
 
         with self.__instances_lock:
-            if name not in self.__instances:
-                raise ValueError("Unknown component instance '{0}'" \
-                                 .format(name))
+            try:
+                # Running instance
+                stored_instance = self.__instances.pop(name)
 
-            stored_instance = self.__instances.pop(name)
+                # Kill it
+                stored_instance.kill()
 
-            # Kill it
-            stored_instance.kill()
+            except KeyError:
+                # Queued instance
+                try:
+                    del self.__waiting_handlers[name]
+
+                except KeyError:
+                    raise ValueError("Unknown component instance '{0}'" \
+                                     .format(name))
 
 
     def register_factory(self, bundle_context, factory):
@@ -725,7 +834,10 @@ class _IPopoService(object):
             return False
 
         with self.__factories_lock:
-            if factory_name not in self.__factories:
+            try:
+                # Remove the factory from the registry
+                factory_class = self.__factories.pop(factory_name)
+            except KeyError:
                 # Unknown factory
                 return False
 
@@ -736,7 +848,7 @@ class _IPopoService(object):
             # Invalidate and delete all components of this factory
             with self.__instances_lock:
                 # Compute the list of __instances to remove
-                to_remove = self.__get_stored_instances_by_factory(factory_name)
+                to_remove = self.__get_stored_instances(factory_name)
 
                 # Remove instances from the registry: avoids dependencies \
                 # update to link against a component from this factory again.
@@ -751,8 +863,16 @@ class _IPopoService(object):
                         # => ignore
                         pass
 
-            # Remove the factory from the registry
-            del self.__factories[factory_name]
+                # Remove waiting component
+                names = [name
+                         for name, (context, _) \
+                                            in self.__waiting_handlers.items()
+                         if context.factory_context.name == factory_name]
+                for name in names:
+                    del self.__waiting_handlers[name]
+
+            # Clear the bundle context of the factory
+            _set_factory_context(factory_class, None)
 
         return True
 
@@ -795,11 +915,26 @@ class _IPopoService(object):
 
         :return: A list of (name, factory name, state) tuples.
         """
+        with self.__instances_lock:
+            return sorted((name, stored_instance.factory_name,
+                           stored_instance.state)
+                          for name, stored_instance in self.__instances.items())
+
+
+    def get_waiting_components(self):
+        """
+        Returns the list of the instances waiting for their handlers
+
+        :return: A list of (name, factory name, missing handlers) tuples
+        """
         with self.__instances_lock:
             result = []
-            for name, stored_instance in self.__instances.items():
-                result.append((name, stored_instance.factory_name,
-                               stored_instance.state))
+            for name, (context, _) in self.__waiting_handlers.items():
+                # Compute missing handlers
+                missing = set(context.factory_context.get_handlers_ids())
+                missing.difference_update(self._handlers.keys())
+
+                result.append((name, context.factory_context.name, missing))
 
             result.sort()
             return result
@@ -864,7 +999,7 @@ class _IPopoService(object):
                                         handlers_const.KIND_SERVICE_PROVIDER):
                     svc_ref = handler.get_service_reference()
                     if svc_ref is not None:
-                        svc_id = svc_ref.get_property(pelix.SERVICE_ID)
+                        svc_id = svc_ref.get_property(SERVICE_ID)
                         result["services"][svc_id] = svc_ref
 
                 # Dependencies
@@ -929,7 +1064,23 @@ class _IPopoService(object):
 
     def get_factory_details(self, name):
         """
-        Retrieves details about the given factory
+        Retrieves a dictionary with details about the given factory
+
+        * name: The factory name
+        * bundle: The Bundle object of the bundle providing the factory
+        * properties: Copy of the components properties defined by the factory
+        * requirements: List of the requirements defined by the factory
+          * id: Requirement ID (field where it is injected)
+          * specification: Specification of the required service
+          * aggregate: If True, multiple services will be injected
+          * optional: If True, the requirement is optional
+        * services: List of the specifications of the services provided by
+          components of this factory
+        * handlers: Dictionary of the non-built-in handlers required by this
+          factory. The dictionary keys are handler IDs, and it contains a tuple
+          with:
+          * A copy of the configuration of the handler (0)
+          * A flag indicating if the handler is present or not
 
         :param name: The name of a factory
         :return: A dictionary describing the factory
@@ -956,25 +1107,27 @@ class _IPopoService(object):
 
             # Requirements (list of dictionaries)
             reqs = result["requirements"] = []
-            for field, requirement \
-            in context.get_handler(constants.HANDLER_REQUIRES).items():
-                req = {}
-                # ID = Field name
-                req["id"] = field
-                req["specification"] = requirement.specification
-                req["aggregate"] = requirement.aggregate
-                req["optional"] = requirement.optional
+            handler_requires = context.get_handler(constants.HANDLER_REQUIRES)
+            if handler_requires is not None:
+                for field, requirement in handler_requires.items():
+                    req = {}
+                    # ID = Field name
+                    req["id"] = field
+                    req["specification"] = requirement.specification
+                    req["aggregate"] = requirement.aggregate
+                    req["optional"] = requirement.optional
 
-                # Give the string representation of the original LDAP filter
-                req["filter"] = requirement.original_filter
+                    # Give the string representation of the original LDAP filter
+                    req["filter"] = requirement.original_filter
 
-                reqs.append(req)
+                    reqs.append(req)
 
             # Provided services (list of list of specifications)
             svc = result["services"] = []
-            for specs_controller \
-            in context.get_handler(constants.HANDLER_PROVIDES):
-                svc.append(specs_controller[0])
+            handler_provides = context.get_handler(constants.HANDLER_PROVIDES)
+            if handler_provides is not None:
+                for specs_controller in handler_provides:
+                    svc.append(specs_controller[0])
 
             # Other handlers
             handlers = set(context.get_handlers_ids())
@@ -982,16 +1135,16 @@ class _IPopoService(object):
                                         constants.HANDLER_PROVIDES,
                                         constants.HANDLER_REQUIRES))
 
-            if handlers:
-                result["handlers"] = dict((handler,
-                                           copy.deepcopy(
-                                                 context.get_handler(handler)))
-                                          for handler in handlers)
+            handlers_dict = result["handlers"] = {}
+            for handler in handlers:
+                handlers_dict[handler] = copy.deepcopy(\
+                                                context.get_handler(handler))
 
             return result
 
 # ------------------------------------------------------------------------------
 
+@BundleActivator
 class _IPopoActivator(object):
     """
     The iPOPO bundle activator for Pelix
@@ -1012,8 +1165,6 @@ class _IPopoActivator(object):
 
         :param context: The bundle context
         """
-        assert isinstance(context, BundleContext)
-
         # Automatically install handlers bundles
         for handler in BUILTIN_HANDLERS:
             try:
@@ -1021,15 +1172,14 @@ class _IPopoActivator(object):
                 bundle.start()
                 self._bundles.append(bundle)
 
-            except pelix.BundleException as ex:
+            except BundleException as ex:
                 _logger.error("Error installing handler %s: %s", handler, ex)
 
 
         # Register the iPOPO service
         self._service = _IPopoService(context)
-        self._registration = context.register_service(\
-                                        constants.IPOPO_SERVICE_SPECIFICATION, \
-                                        self._service, {})
+        self._registration = context.register_service(constants.SERVICE_IPOPO,
+                                                      self._service, {})
 
         # Register as a bundle listener
         context.add_bundle_listener(self._service)
@@ -1053,8 +1203,6 @@ class _IPopoActivator(object):
 
         :param context: The bundle context
         """
-        assert isinstance(context, BundleContext)
-
         # The service is not in the "run" mode anymore
         self._service._stop()
 
@@ -1078,8 +1226,3 @@ class _IPopoActivator(object):
         # Clean up references
         self._registration = None
         self._service = None
-
-# ------------------------------------------------------------------------------
-
-# The activator instance
-activator = _IPopoActivator()
