@@ -4,14 +4,14 @@
 Defines the iPOPO decorators classes to manipulate component factory classes
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Defines the iPOPO decorators classes to manipulate component factory classes
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -36,7 +36,7 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 # Pelix modules
-from pelix.utilities import is_string
+from pelix.utilities import is_string, to_iterable
 from pelix.ipopo.contexts import FactoryContext, Requirement
 import pelix.ipopo.constants as constants
 
@@ -84,6 +84,36 @@ def is_from_parent(cls, attribute_name, value=None):
     return False
 
 
+def get_factory_context(cls):
+    """
+    Retrieves the factory context object associated to a factory. Creates it
+    if needed
+
+    :param cls: The factory class
+    :return: The factory class context
+    """
+    context = getattr(cls, constants.IPOPO_FACTORY_CONTEXT, None)
+
+    if context is None:
+        # Class not yet manipulated
+        context = FactoryContext()
+
+    elif is_from_parent(cls, constants.IPOPO_FACTORY_CONTEXT):
+        # Create a copy the context
+        context = context.copy(True)
+
+        # * Manipulation has not been applied yet
+        context.completed = False
+
+    else:
+        # Nothing special to do
+        return context
+
+    # Context has been created or copied, inject the new bean
+    setattr(cls, constants.IPOPO_FACTORY_CONTEXT, context)
+    return context
+
+
 def get_method_description(method):
     """
     Retrieves a description of the given method. If possible, the description
@@ -147,41 +177,6 @@ def validate_method_arity(method, *needed_args):
 
 # ------------------------------------------------------------------------------
 
-def _get_factory_context(cls):
-    """
-    Retrieves the factory context object associated to a factory. Creates it
-    if needed
-
-    :param cls: The factory class
-    :return: The factory class context
-    """
-    context = getattr(cls, constants.IPOPO_FACTORY_CONTEXT, None)
-
-    if context is None:
-        # Class not yet manipulated
-        context = FactoryContext()
-
-    elif is_from_parent(cls, constants.IPOPO_FACTORY_CONTEXT):
-        # Create a copy the context
-        context = context.copy()
-
-        # Clear the values that must not be inherited:
-        # * Provided services
-        # FIXME: do it in a better/generic way
-        context.set_handler(constants.HANDLER_PROVIDES, [])
-
-        # * Manipulation has not been applied yet
-        context.completed = False
-
-    else:
-        # Nothing special to do
-        return context
-
-    # Context has been created or copied, inject the new bean
-    setattr(cls, constants.IPOPO_FACTORY_CONTEXT, context)
-    return context
-
-
 def _ipopo_setup_callback(cls, context):
     """
     Sets up the class _callback dictionary
@@ -271,18 +266,18 @@ def _ipopo_setup_field_callback(cls, context):
         # child will see the attribute -> Don't remove it
 
         # Store the call backs
-        for kind, field in method_callbacks:
+        for kind, field, if_valid in method_callbacks:
             fields_cbs = callbacks.setdefault(field, {})
 
             if kind in fields_cbs and \
-            not is_from_parent(cls, fields_cbs[kind].__name__):
+            not is_from_parent(cls, fields_cbs[kind][0].__name__):
                 _logger.warning("Redefining the callback %s in '%s'. " \
                                 "Previous callback : '%s' (%s). " \
                                 "New callback : %s", kind, name,
-                                fields_cbs[kind].__name__,
-                                fields_cbs[kind], function)
+                                fields_cbs[kind][0].__name__,
+                                fields_cbs[kind][0], function)
 
-            fields_cbs[kind] = function
+            fields_cbs[kind] = (function, if_valid)
 
     # Update the factory context
     context.field_callbacks.clear()
@@ -414,21 +409,15 @@ class Instantiate(object):
         :raise TypeError: The given object is not a class
         """
         if not inspect.isclass(factory_class):
-            raise TypeError("@ComponentFactory can decorate only classes, " \
+            raise TypeError("@Instantiate can decorate only classes, " \
                             "not '{0}'".format(type(factory_class).__name__))
 
-        instances = getattr(factory_class, constants.IPOPO_INSTANCES, None)
+        # Store the instance in the factory context
+        context = get_factory_context(factory_class)
+        try:
+            context.add_instance(self.__name, self.__properties)
 
-        if instances is None or \
-            is_from_parent(factory_class, constants.IPOPO_INSTANCES):
-            # No instances for this particular class
-            instances = {}
-            setattr(factory_class, constants.IPOPO_INSTANCES, instances)
-
-        if self.__name not in instances:
-            instances[self.__name] = self.__properties
-
-        else:
+        except NameError:
             _logger.warning("Component '%s' defined twice, new definition "
                             "ignored", self.__name)
 
@@ -440,20 +429,16 @@ class ComponentFactory(object):
     """
     Decorator that sets up a component factory class
     """
-
-    NON_INHERITABLE_FIELDS = (constants.IPOPO_INSTANCES,)
-    """
-    Non inheritable fields, i.e. to remove during the manipulation of child
-    classes, if needed
-    """
-
-    def __init__(self, name=None):
+    def __init__(self, name=None, excluded=None):
         """
         Sets up the decorator
 
         :param name: Name of the component factory
+        :param excluded: List of IDs of handlers which configuration must not
+                         be inherited from the parent class
         """
         self.__factory_name = name
+        self.__excluded_inheritance = to_iterable(excluded)
 
 
     def __call__(self, factory_class):
@@ -469,7 +454,7 @@ class ComponentFactory(object):
                             "not '{0}'".format(type(factory_class).__name__))
 
         # Get the factory context
-        context = _get_factory_context(factory_class)
+        context = get_factory_context(factory_class)
 
         # Test if a manipulation has already been applied
         if not context.completed:
@@ -478,19 +463,16 @@ class ComponentFactory(object):
                 self.__factory_name = factory_class.__name__ + "Factory"
 
             # Manipulate the class...
+
+            # Update the factory context
             context.name = self.__factory_name
+            context.inherit_handlers(self.__excluded_inheritance)
             context.completed = True
 
             # Find callbacks
             _ipopo_setup_callback(factory_class, context)
             _ipopo_setup_field_callback(factory_class, context)
 
-            # Clean up inherited fields, to avoid weird behavior
-            for field in ComponentFactory.NON_INHERITABLE_FIELDS:
-                if is_from_parent(factory_class, field):
-                    # Set inherited fields to None
-                    setattr(factory_class, field, None)
-
             # Store the factory context in its field
             setattr(factory_class, constants.IPOPO_FACTORY_CONTEXT, context)
 
@@ -517,6 +499,9 @@ class Property(object):
 
     Defines a component property.
     """
+    HANDLER_ID = constants.HANDLER_PROPERTY
+    """ ID of the handler configured by this decorator """
+
     def __init__(self, field=None, name=None, value=None):
         """
         Sets up the property
@@ -566,7 +551,7 @@ class Property(object):
                             .format(type(clazz).__name__))
 
         # Get the factory context
-        context = _get_factory_context(clazz)
+        context = get_factory_context(clazz)
         if context.completed:
             # Do nothing if the class has already been manipulated
             _logger.warning("@Property: Already manipulated class: %s",
@@ -580,7 +565,7 @@ class Property(object):
         context.properties_fields[self.__field] = self.__name
 
         # Mark the handler in the factory context
-        context.set_handler(constants.HANDLER_PROPERTY, None)
+        context.set_handler(self.HANDLER_ID, None)
 
         # Inject a property in the class. The property will call an instance
         # level getter / setter, injected by iPOPO after the instance creation
@@ -634,7 +619,10 @@ class Provides(object):
 
     Defines an interface exported by a component.
     """
-    def __init__(self, specifications=None, controller=None):
+    HANDLER_ID = constants.HANDLER_PROVIDES
+    """ ID of the handler configured by this decorator """
+
+    def __init__(self, specifications, controller=None):
         """
         Sets up a provided service.
         A service controller can be defined to enable or disable the service.
@@ -676,7 +664,7 @@ class Provides(object):
                             .format(type(clazz).__name__))
 
         # Get the factory context
-        context = _get_factory_context(clazz)
+        context = get_factory_context(clazz)
         if context.completed:
             # Do nothing if the class has already been manipulated
             _logger.warning("@Provides: Already manipulated class: %s",
@@ -690,7 +678,7 @@ class Provides(object):
                 filtered_specs.append(spec)
 
         # Store the service information
-        config = context.get_handler(constants.HANDLER_PROVIDES, [])
+        config = context.set_handler_default(self.HANDLER_ID, [])
         config.append((filtered_specs, self.__controller))
 
         if self.__controller:
@@ -715,10 +703,13 @@ class Requires(object):
     """
     @Requires decorator
 
-    Defines a required component
+    Defines a required service
     """
-    def __init__(self, field="", specification="", aggregate=False, \
-                 optional=False, spec_filter=None):
+    HANDLER_ID = constants.HANDLER_REQUIRES
+    """ ID of the handler configured by this decorator """
+
+    def __init__(self, field, specification, aggregate=False, optional=False, \
+                 spec_filter=None):
         """
         Sets up the requirement
 
@@ -769,7 +760,7 @@ class Requires(object):
                             clazz.__name__, self.__field)
 
         # Set up the property in the class
-        context = _get_factory_context(clazz)
+        context = get_factory_context(clazz)
         if context.completed:
             # Do nothing if the class has already been manipulated
             _logger.warning("@Requires: Already manipulated class: %s",
@@ -777,7 +768,7 @@ class Requires(object):
             return clazz
 
         # Store the requirement information
-        config = context.get_handler(constants.HANDLER_REQUIRES, {})
+        config = context.set_handler_default(self.HANDLER_ID, {})
         config[self.__field] = self.__requirement
 
         # Inject the field
@@ -787,6 +778,97 @@ class Requires(object):
 
 # ------------------------------------------------------------------------------
 
+class RequiresMap(object):
+    """
+    @RequiresMap decorator
+
+    Defines a required service, injected in a dictionary
+    """
+    HANDLER_ID = constants.HANDLER_REQUIRES_MAP
+    """ ID of the handler configured by this decorator """
+
+    def __init__(self, field, specification, key, allow_none=False,
+                 aggregate=False, optional=False, spec_filter=None):
+        """
+        Sets up the requirement
+
+        :param field: The injected field
+        :param specification: The injected service specification
+        :param key: Name of the service property to use as a dictionary key
+        :param allow_none: If True, inject services with a None property value
+        :param aggregate: If true, injects a list
+        :param optional: If true, this injection is optional
+        :param spec_filter: An LDAP query to filter injected services upon their
+                            properties
+        :raise TypeError: A parameter has an invalid type
+        :raise ValueError: An error occurred while parsing the filter or an
+                           argument is incorrect
+        """
+        # Check if field is valid
+        if not field:
+            raise ValueError("Empty field name.")
+
+        if not is_string(field):
+            raise TypeError("The field name must be a string, not {0}" \
+                            .format(type(field).__name__))
+
+        if ' ' in field:
+            raise ValueError("Field name can't contain spaces.")
+
+        self.__field = field
+
+        # Be sure that there is only one required specification
+        specifications = _get_specifications(specification)
+        self.__multi_specs = len(specifications) > 1
+
+        # Check if key is valid
+        if not key:
+            raise ValueError("No property key given")
+
+        # Store the flags
+        self.__key = key
+        self.__allow_none = allow_none
+
+        # Construct the requirement object
+        self.__requirement = Requirement(specifications[0],
+                                         aggregate, optional, spec_filter)
+
+    def __call__(self, clazz):
+        """
+        Adds the requirement to the class iPOPO field
+
+        :param clazz: The class to decorate
+        :return: The decorated class
+        :raise TypeError: If *clazz* is not a type
+        """
+        if not inspect.isclass(clazz):
+            raise TypeError("@RequiresMap can decorate only classes, not '{0}'"\
+                            .format(type(clazz).__name__))
+
+        if self.__multi_specs:
+            _logger.warning("Only one specification can be required: %s -> %s",
+                            get_method_description(clazz), self.__field)
+
+        # Set up the property in the class
+        context = get_factory_context(clazz)
+        if context.completed:
+            # Do nothing if the class has already been manipulated
+            _logger.warning("@RequiresMap: Already manipulated class: %s",
+                            get_method_description(clazz))
+            return clazz
+
+        # Store the requirement information
+        config = context.set_handler_default(self.HANDLER_ID, {})
+        config[self.__field] = (self.__requirement,
+                                self.__key, self.__allow_none)
+
+        # Inject the field
+        setattr(clazz, self.__field, None)
+
+        return clazz
+
+# ------------------------------------------------------------------------------
+
 class BindField(object):
     """
     BindField callback decorator, called when a component is bound to a
@@ -813,13 +895,15 @@ class BindField(object):
 
     Exceptions raised by a bind callback are ignored.
     """
-    def __init__(self, field):
+    def __init__(self, field, if_valid=False):
         """
         Sets up the decorator
 
         :param field: Field associated to the binding
+        :param if_valid: Call the method only if the component is valid
         """
         self._field = field
+        self._if_valid = if_valid
 
 
     def __call__(self, method):
@@ -838,7 +922,7 @@ class BindField(object):
 
         _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
                              (constants.IPOPO_CALLBACK_BIND_FIELD,
-                              self._field))
+                              self._field, self._if_valid))
         return method
 
 
@@ -862,13 +946,15 @@ class UpdateField(object):
 
     Exceptions raised by an update callback are ignored.
     """
-    def __init__(self, field):
+    def __init__(self, field, if_valid=False):
         """
         Sets up the decorator
 
         :param field: Field associated to the binding
+        :param if_valid: Call the method only if the component is valid
         """
         self._field = field
+        self._if_valid = if_valid
 
 
     def __call__(self, method):
@@ -888,7 +974,7 @@ class UpdateField(object):
 
         _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
                              (constants.IPOPO_CALLBACK_UPDATE_FIELD,
-                              self._field))
+                              self._field, self._if_valid))
         return method
 
 
@@ -916,13 +1002,15 @@ class UnbindField(object):
 
     Exceptions raised by an unbind callback are ignored.
     """
-    def __init__(self, field):
+    def __init__(self, field, if_valid=False):
         """
         Sets up the decorator
 
         :param field: Field associated to the binding
+        :param if_valid: Call the method only if the component is valid
         """
         self._field = field
+        self._if_valid = if_valid
 
 
     def __call__(self, method):
@@ -941,7 +1029,7 @@ class UnbindField(object):
 
         _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
                              (constants.IPOPO_CALLBACK_UNBIND_FIELD,
-                              self._field))
+                              self._field, self._if_valid))
         return method
 
 # ------------------------------------------------------------------------------
