@@ -4,14 +4,14 @@
 Service providing handler
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Service providing handler
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -36,7 +36,7 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 # Pelix beans
-from pelix.constants import BundleException
+from pelix.constants import BundleActivator, BundleException
 
 # iPOPO constants
 import pelix.ipopo.constants as ipopo_constants
@@ -71,6 +71,7 @@ class _HandlerFactory(constants.HandlerFactory):
                 for specs, controller in provides]
 
 
+@BundleActivator
 class _Activator(object):
     """
     The bundle activator
@@ -105,9 +106,6 @@ class _Activator(object):
         self._registration.unregister()
         self._registration = None
 
-# Declare the activator
-activator = _Activator()
-
 # ------------------------------------------------------------------------------
 
 class ServiceRegistrationHandler(constants.ServiceProviderHandler):
