@@ -4,14 +4,14 @@
 Instance manager class definition
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Instance manager class definition
 """
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -165,13 +165,20 @@ class StoredInstance(object):
             self.check_lifecycle()
 
 
-    def update(self, dependency, svc, svc_ref, old_properties):
+    def update(self, dependency, svc, svc_ref, old_properties, new_value=False):
         """
         Called by a dependency manager when the properties of an injected
         dependency have been updated.
+
+        :param dependency: The dependency handler
+        :param svc: The injected service
+        :param svc_ref: The reference of the injected service
+        :param old_properties: Previous properties of the dependency
+        :param new_value: If True, inject the new value of the handler
         """
         with self._lock:
-            self.__update_binding(dependency, svc, svc_ref, old_properties)
+            self.__update_binding(dependency, svc, svc_ref, old_properties,
+                                  new_value)
             self.check_lifecycle()
 
 
@@ -245,7 +252,8 @@ class StoredInstance(object):
             for handlers_list in self._handlers.values():
                 result.update(handlers_list)
 
-            return result
+            # Always return a list
+            return list(result)
 
 
     def check_lifecycle(self):
@@ -438,19 +446,18 @@ class StoredInstance(object):
         :return: The callback result, or None
         :raise Exception: Something went wrong
         """
-        with self._lock:
-            comp_callback = self.context.get_callback(event)
-            if not comp_callback:
-                # No registered callback
-                return True
+        comp_callback = self.context.get_callback(event)
+        if not comp_callback:
+            # No registered callback
+            return True
 
-            # Call it
-            result = comp_callback(self.instance, *args, **kwargs)
-            if result is None:
-                # Special case, if the call back returns nothing
-                return True
+        # Call it
+        result = comp_callback(self.instance, *args, **kwargs)
+        if result is None:
+            # Special case, if the call back returns nothing
+            return True
 
-            return result
+        return result
 
 
     def __field_callback(self, field, event, *args, **kwargs):
@@ -462,19 +469,26 @@ class StoredInstance(object):
         :return: The callback result, or None
         :raise Exception: Something went wrong
         """
-        with self._lock:
-            comp_callback = self.context.get_field_callback(field, event)
-            if not comp_callback:
-                # No registered callback
-                return True
+        # Get the field callback info
+        cb_info = self.context.get_field_callback(field, event)
+        if not cb_info:
+            # No registered callback
+            return True
 
-            # Call it
-            result = comp_callback(self.instance, field, *args, **kwargs)
-            if result is None:
-                # Special case, if the call back returns nothing
-                return True
+        # Extract information
+        callback, if_valid = cb_info
 
-            return result
+        if if_valid and self.state != StoredInstance.VALID:
+            # Don't call the method if the component state isn't satisfying
+            return True
+
+        # Call it
+        result = callback(self.instance, field, *args, **kwargs)
+        if result is None:
+            # Special case, if the call back returns nothing
+            return True
+
+        return result
 
 
     def __safe_callback(self, event, *args, **kwargs):
@@ -485,34 +499,33 @@ class StoredInstance(object):
         :param event: An event (IPOPO_CALLBACK_VALIDATE, ...)
         :return: The callback result, or None
         """
-        with self._lock:
-            if self.state == StoredInstance.KILLED:
-                # Invalid state
-                return None
+        if self.state == StoredInstance.KILLED:
+            # Invalid state
+            return None
 
-            try:
-                return self.__callback(event, *args, **kwargs)
+        try:
+            return self.__callback(event, *args, **kwargs)
 
-            except FrameworkException as ex:
-                # Important error
-                self._logger.exception("Critical error calling back %s: %s",
-                                       self.name, ex)
+        except FrameworkException as ex:
+            # Important error
+            self._logger.exception("Critical error calling back %s: %s",
+                                   self.name, ex)
 
-                # Kill the component
-                self._ipopo_service.kill(self.name)
+            # Kill the component
+            self._ipopo_service.kill(self.name)
 
-                if ex.needs_stop:
-                    # Framework must be stopped...
-                    self._logger.error("%s said that the Framework must be "
-                                       "stopped.", self.name)
-                    self.bundle_context.get_bundle(0).stop()
-                return False
+            if ex.needs_stop:
+                # Framework must be stopped...
+                self._logger.error("%s said that the Framework must be "
+                                   "stopped.", self.name)
+                self.bundle_context.get_bundle(0).stop()
+            return False
 
-            except:
-                self._logger.exception("Component '%s' : error calling "
-                                       "callback method for event %s",
-                                       self.name, event)
-                return False
+        except:
+            self._logger.exception("Component '%s' : error calling "
+                                   "callback method for event %s",
+                                   self.name, event)
+            return False
 
 
     def __safe_field_callback(self, field, event, *args, **kwargs):
@@ -524,34 +537,33 @@ class StoredInstance(object):
         :param event: A field event (IPOPO_CALLBACK_BIND_FIELD, ...)
         :return: The callback result, or None
         """
-        with self._lock:
-            if self.state == StoredInstance.KILLED:
-                # Invalid state
-                return None
+        if self.state == StoredInstance.KILLED:
+            # Invalid state
+            return None
 
-            try:
-                return self.__field_callback(field, event, *args, **kwargs)
+        try:
+            return self.__field_callback(field, event, *args, **kwargs)
 
-            except FrameworkException as ex:
-                # Important error
-                self._logger.exception("Critical error calling back %s: %s",
-                                       self.name, ex)
+        except FrameworkException as ex:
+            # Important error
+            self._logger.exception("Critical error calling back %s: %s",
+                                   self.name, ex)
 
-                # Kill the component
-                self._ipopo_service.kill(self.name)
+            # Kill the component
+            self._ipopo_service.kill(self.name)
 
-                if ex.needs_stop:
-                    # Framework must be stopped...
-                    self._logger.error("%s said that the Framework must be "
-                                       "stopped.", self.name)
-                    self.bundle_context.get_bundle(0).stop()
-                return False
+            if ex.needs_stop:
+                # Framework must be stopped...
+                self._logger.error("%s said that the Framework must be "
+                                   "stopped.", self.name)
+                self.bundle_context.get_bundle(0).stop()
+            return False
 
-            except:
-                self._logger.exception("Component '%s' : error calling "
-                                       "callback method for event %s",
-                                       self.name, event)
-                return False
+        except:
+            self._logger.exception("Component '%s' : error calling "
+                                   "callback method for event %s",
+                                   self.name, event)
+            return False
 
 
     def __safe_handler_callback(self, handler, method_name, *args, **kwargs):
@@ -584,15 +596,23 @@ class StoredInstance(object):
         none_as_true = kwargs.pop('none_as_true', False)
 
         # Get the method for each handler
-        result = None
-        method = getattr(handler, method_name, None)
-        if method is not None:
+        try:
+            method = getattr(handler, method_name)
+
+        except AttributeError:
+            # Method not found
+            result = None
+
+        else:
             try:
                 # Call it
                 result = method(*args, **kwargs)
 
             except Exception as ex:
-                # Log errors
+                # No result
+                result = None
+
+                # Log error
                 self._logger.exception("Error calling handler '%s': %s",
                                        handler, ex)
 
@@ -602,7 +622,7 @@ class StoredInstance(object):
 
         if only_boolean:
             # Convert to a boolean result
-            result = only_boolean and result
+            return bool(result)
 
         return result
 
@@ -628,46 +648,45 @@ class StoredInstance(object):
                        behavior of the call
         :return: True if all handlers returned True (or None), else False
         """
-        with self._lock:
-            if self.state == StoredInstance.KILLED:
-                # Nothing to do
-                return False
+        if self.state == StoredInstance.KILLED:
+            # Nothing to do
+            return False
 
-            # Behavior flags
-            exception_as_error = kwargs.pop('exception_as_error', False)
-            break_on_false = kwargs.pop('break_on_false', False)
+        # Behavior flags
+        exception_as_error = kwargs.pop('exception_as_error', False)
+        break_on_false = kwargs.pop('break_on_false', False)
 
-            result = True
-            for handler in self.get_handlers():
-                # Get the method for each handler
+        result = True
+        for handler in self.get_handlers():
+            # Get the method for each handler
+            try:
+                method = getattr(handler, method_name)
+
+            except AttributeError:
+                # Ignore missing methods
+                pass
+
+            else:
                 try:
-                    method = getattr(handler, method_name)
+                    # Call it
+                    res = method(*args, **kwargs)
+                    if res is not None and not res:
+                        # Ignore 'None' results
+                        result = False
 
-                except AttributeError:
-                    # Ignore missing methods
-                    pass
+                except Exception as ex:
+                    # Log errors
+                    self._logger.exception("Error calling handler '%s': %s",
+                                           handler, ex)
 
-                else:
-                    try:
-                        # Call it
-                        res = method(*args, **kwargs)
-                        if res is not None and not res:
-                            # Ignore 'None' results
-                            result = False
+                    # We can consider exceptions as errors or ignore them
+                    result &= not exception_as_error
 
-                    except Exception as ex:
-                        # Log errors
-                        self._logger.exception("Error calling handler '%s': %s",
-                                               handler, ex)
+                if not handler and break_on_false:
+                    # The loop can stop here
+                    break
 
-                        # We can consider exceptions as errors or ignore them
-                        result &= not exception_as_error
-
-                    if not handler and break_on_false:
-                        # The loop can stop here
-                        break
-
-            return result
+        return result
 
 
     def __set_binding(self, dependency, service, reference):
@@ -678,21 +697,19 @@ class StoredInstance(object):
         :param service: The injected service
         :param reference: The reference of the injected service
         """
-        with self._lock:
-            # Set the value
-            setattr(self.instance, dependency.get_field(),
-                    dependency.get_value())
+        # Set the value
+        setattr(self.instance, dependency.get_field(), dependency.get_value())
 
-            # Call the component back
-            self.__safe_callback(constants.IPOPO_CALLBACK_BIND,
-                                 service, reference)
+        # Call the component back
+        self.__safe_callback(constants.IPOPO_CALLBACK_BIND, service, reference)
 
-            self.__safe_field_callback(dependency.get_field(),
-                                       constants.IPOPO_CALLBACK_BIND_FIELD,
-                                       service, reference)
+        self.__safe_field_callback(dependency.get_field(),
+                                   constants.IPOPO_CALLBACK_BIND_FIELD,
+                                   service, reference)
 
 
-    def __update_binding(self, dependency, service, reference, old_properties):
+    def __update_binding(self, dependency, service, reference, old_properties,
+                         new_value):
         """
         Calls back component binding and field binding methods when the
         properties of an injected dependency have been updated.
@@ -701,15 +718,20 @@ class StoredInstance(object):
         :param service: The injected service
         :param reference: The reference of the injected service
         :param old_properties: Previous properties of the dependency
+        :param new_value: If True, inject the new value of the handler
         """
-        with self._lock:
-            # Call the component back
-            self.__safe_field_callback(dependency.get_field(),
-                                       constants.IPOPO_CALLBACK_UPDATE_FIELD,
-                                       service, reference, old_properties)
+        if new_value:
+            # Set the value
+            setattr(self.instance, dependency.get_field(),
+                    dependency.get_value())
 
-            self.__safe_callback(constants.IPOPO_CALLBACK_UPDATE,
-                                 service, reference, old_properties)
+        # Call the component back
+        self.__safe_field_callback(dependency.get_field(),
+                                   constants.IPOPO_CALLBACK_UPDATE_FIELD,
+                                   service, reference, old_properties)
+
+        self.__safe_callback(constants.IPOPO_CALLBACK_UPDATE, service,
+                             reference, old_properties)
 
 
     def __unset_binding(self, dependency, service, reference):
@@ -720,18 +742,16 @@ class StoredInstance(object):
         :param service: The injected service
         :param reference: The reference of the injected service
         """
-        with self._lock:
-            # Call the component back
-            self.__safe_field_callback(dependency.get_field(),
-                                       constants.IPOPO_CALLBACK_UNBIND_FIELD,
-                                       service, reference)
+        # Call the component back
+        self.__safe_field_callback(dependency.get_field(),
+                                   constants.IPOPO_CALLBACK_UNBIND_FIELD,
+                                   service, reference)
 
-            self.__safe_callback(constants.IPOPO_CALLBACK_UNBIND,
-                                 service, reference)
+        self.__safe_callback(constants.IPOPO_CALLBACK_UNBIND, service,
+                             reference)
 
-            # Update the injected field
-            setattr(self.instance, dependency.get_field(),
-                    dependency.get_value())
+        # Update the injected field
+        setattr(self.instance, dependency.get_field(), dependency.get_value())
 
-            # Unget the service
-            self.bundle_context.unget_service(reference)
+        # Unget the service
+        self.bundle_context.unget_service(reference)
