@@ -12,14 +12,14 @@ previous service properties (if the event is an update).
 when using remote services on the local host only.
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.2
+:version: 0.3.0
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -35,7 +35,7 @@ when using remote services on the local host only.
 """
 
 # Module version
-__version_info__ = (0, 2, 0)
+__version_info__ = (0, 3, 0)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -62,14 +62,6 @@ import socket
 import struct
 import threading
 
-try:
-    # Python 3
-    import http.client as httplib
-
-except ImportError:
-    # Python 2
-    import httplib
-
 # ------------------------------------------------------------------------------
 
 _logger = logging.getLogger(__name__)
@@ -96,6 +88,9 @@ if os.name == "nt":
 
             # Prepare structure
             class sockaddr_in6(ctypes.Structure):
+                """
+                Definition of the C structure 'sockaddr_in6'
+                """
                 _fields_ = [("sin6_family", ctypes.c_short),
                             ("sin6_port", ctypes.c_ushort),
                             ("sin6_flowinfo", ctypes.c_ulong),
@@ -276,13 +271,11 @@ def close_multicast_socket(sock, address):
 # ------------------------------------------------------------------------------
 
 @ComponentFactory(pelix.remote.FACTORY_DISCOVERY_MULTICAST)
-@Provides(pelix.remote.SERVICE_ENDPOINT_LISTENER)
-@Requires("_dispatcher", pelix.remote.SERVICE_DISPATCHER)
+@Provides(pelix.remote.SERVICE_EXPORT_ENDPOINT_LISTENER)
 @Requires('_access', pelix.remote.SERVICE_DISPATCHER_SERVLET)
 @Requires("_registry", pelix.remote.SERVICE_REGISTRY)
 @Property("_group", "multicast.group", "239.0.0.1")
 @Property("_port", "multicast.port", 42000)
-@Property("_listener_flag", pelix.remote.PROP_LISTEN_EXPORTED, True)
 class MulticastDiscovery(object):
     """
     Remote services discovery and notification using multicast packets
@@ -291,11 +284,7 @@ class MulticastDiscovery(object):
         """
         Sets up the component
         """
-        # End point listener flag
-        self._listener_flag = True
-
         # End points registry
-        self._dispatcher = None
         self._registry = None
 
         # Dispatcher access
@@ -315,28 +304,58 @@ class MulticastDiscovery(object):
         self._thread = None
 
 
-    def _make_endpoint_dict(self, event, endpoint):
+    def __make_basic_dict(self, event):
         """
-        Converts the end point into an event packet dictionary
+        Prepares basic common information contained into an event packet
+        (access, framework UID, event type)
 
-        :param event: The kind of event: add, update, remove
-        :param endpoint: An end point description object
+        :param event: The kind of event
         :return: A dictionary
         """
         # Get the dispatcher servlet access
         access = self._access.get_access()
 
         # Make the event packet content
-        packet = {"sender": self._fw_uid,  # Framework UID
-                  "event": event,  # Kind of event
-                  "uid": endpoint.uid,  # Endpoint UID
-                  "access": {"port": access[0],  # Access to the dispatcher
-                             "path": access[1]}  # servlet
-                  }
+        return {"sender": self._fw_uid,  # Framework UID
+                "event": event,  # Kind of event
+                "access": {"port": access[0],  # Access to the dispatcher
+                           "path": access[1]}  # servlet
+                }
 
+
+    def _make_endpoint_dict(self, event, endpoint):
+        """
+        Prepares an event packet containing a single endpoint
+
+        :param event: The kind of event (update, remove)
+        :param endpoint: An ExportEndpoint bean
+        :return: A dictionary
+        """
+        # Basic packet information
+        packet = self.__make_basic_dict(event)
+
+        # Add endpoint information
+        packet['uid'] = endpoint.uid
         if event == "update":
             # Give the new end point properties
-            packet["new_properties"] = endpoint.reference.get_properties()
+            packet["new_properties"] = endpoint.make_import_properties()
+
+        return packet
+
+
+    def _make_endpoints_dict(self, event, endpoints):
+        """
+        Prepares an event packet containing multiple endpoints
+
+        :param event: The kind of event (add)
+        :param endpoints: A list of ExportEndpoint beans
+        :return: A dictionary
+        """
+        # Basic packet information
+        packet = self.__make_basic_dict(event)
+
+        # Add endpoints information
+        packet['uids'] = [endpoint.uid for endpoint in endpoints]
 
         return packet
 
@@ -364,27 +383,17 @@ class MulticastDiscovery(object):
         """
         Sends a discovery packet, requesting others to indicate their services
         """
-        # Get the dispatcher servlet access
-        access = self._access.get_access()
-
-        # Make the discovery packet content
-        data = {"event": "discovery",  # Discovery packet
-                "sender": self._fw_uid,  # Framework UID
-                "access": {"port": access[0],  # Access to the dispatcher
-                           "path": access[1]}  # servlet
-                }
-
         # Send a JSON request
-        data = json.dumps(data)
+        data = json.dumps(self.__make_basic_dict("discovery"))
         self.__send_packet(data)
 
 
-    def endpoint_added(self, endpoint):
+    def endpoints_added(self, endpoints):
         """
-        A new service is exported
+        Multiple endpoints have been created
         """
         # Send a JSON event
-        data = json.dumps(self._make_endpoint_dict("add", endpoint))
+        data = json.dumps(self._make_endpoints_dict("add", endpoints))
         self.__send_packet(data)
 
 
@@ -447,91 +456,31 @@ class MulticastDiscovery(object):
         """
         # Get the event
         event = data['event']
-        endpoint_uid = data['uid']
-        framework_uid = data['sender']
 
         if event == 'add':
             # Store it
-            access = data['access']
-            endpoint = self.grab_endpoint(sender[0], access['port'],
-                                          access['path'], endpoint_uid)
-            if endpoint is not None:
-                self._access.register_endpoint(sender[0], endpoint)
+            port = data['access']['port']
+            path = data['access']['path']
+
+            for uid in data['uids']:
+                # Get the description of the endpoint
+                endpoint = self._access.grab_endpoint(sender[0], port,
+                                                      path, uid)
+                if endpoint is not None:
+                    # Register the endpoint
+                    self._registry.add(endpoint)
 
         elif event == 'remove':
             # Remove it
-            self._registry.remove(endpoint_uid)
+            self._registry.remove(data['uid'])
 
         elif event == 'update':
             # Update it
+            endpoint_uid = data['uid']
             new_properties = data['new_properties']
-            self._access.filter_properties(framework_uid, new_properties)
             self._registry.update(endpoint_uid, new_properties)
 
 
-    def grab_endpoint(self, host, port, path, uid):
-        """
-        Retrieves the description of the end point with the given UID at the
-        given dispatcher servlet.
-        Returns the end point description as a dictionary, or None in case of
-        error.
-        Does not register nor converts the end point.
-
-        :param host: Dispatcher host address
-        :param port: Dispatcher HTTP service port
-        :param path: Path to the dispatcher servlet
-        :param uid: The UID of an end point
-        :return: An end point dictionary or None
-        """
-        # Setup the request URI
-        if path[-1] == '/':
-            path = path[:-1]
-
-        request_path = "{0}/endpoint/{1}".format(path, uid)
-
-        return self.__grab_data(host, port, request_path)
-
-
-    def __grab_data(self, host, port, path):
-        """
-        Sends a HTTP request to the server at (host, port), on the given path.
-        Returns the parsed response.
-        Returns None if the HTTP result is not 200 or in case of error.
-
-        :param host: Dispatcher host address
-        :param port: Dispatcher HTTP service port
-        :param path: Request path
-        :return: The parsed response content, or None
-        """
-        # Request the end points
-        try:
-            conn = httplib.HTTPConnection(host, port)
-            conn.request("GET", path)
-            result = conn.getresponse()
-            data = result.read()
-            conn.close()
-
-        except Exception as ex:
-            _logger.exception("Error accessing the dispatcher servlet: %s", ex)
-            return
-
-        if result.status != 200:
-            # Not a valid result
-            return
-
-        try:
-            # Convert the response to a string
-            data = to_str(data)
-
-            # Parse the JSON result
-            return json.loads(data)
-
-        except ValueError as ex:
-            # Error parsing data
-            _logger.error("Error reading the response of the dispatcher: %s",
-                          ex)
-
-
     def _read_loop(self):
         """
         Reads packets from the socket
