@@ -4,14 +4,14 @@
 Pelix remote services: Imported end points registry
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.1.1
+:version: 0.3.0
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Pelix remote services: Imported end points registry
 """
 
 # Module version
-__version_info__ = (0, 1, 1)
+__version_info__ = (0, 3, 0)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -40,7 +40,7 @@ import pelix.remote
 
 # iPOPO decorators
 from pelix.ipopo.decorators import ComponentFactory, Requires, Provides, \
-    Instantiate, Invalidate
+    Instantiate, Invalidate, Validate, BindField
 
 # Standard library
 import logging
@@ -54,8 +54,8 @@ _logger = logging.getLogger(__name__)
 
 @ComponentFactory('pelix-remote-imports-registry-factory')
 @Provides(pelix.remote.SERVICE_REGISTRY)
-@Requires('_listeners', pelix.remote.SERVICE_ENDPOINT_LISTENER, True, True,
-          "({0}=*)".format(pelix.remote.PROP_LISTEN_IMPORTED))
+@Requires('_listeners', pelix.remote.SERVICE_IMPORT_ENDPOINT_LISTENER,
+          aggregate=True, optional=True)
 @Instantiate('pelix-remote-imports-registry')
 class ImportsRegistry(object):
     """
@@ -68,16 +68,34 @@ class ImportsRegistry(object):
         # Listeners (injected)
         self._listeners = []
 
-        # Framework UID -> End point
+        # Framework UID
+        self._fw_uid = None
+
+        # Framework UID -> [Endpoints]
         self._frameworks = {}
 
-        # End point UID -> End point
+        # End point UID -> Endpoint
         self._registry = {}
 
         # Lock
         self.__lock = threading.Lock()
 
 
+    @BindField('_listeners', if_valid=True)
+    def _bind_listener(self, field, listener, svc_ref):
+        """
+        New listener bound
+        """
+        with self.__lock:
+            # Late listener
+            for endpoint in self._registry.values():
+                try:
+                    listener.endpoint_added(endpoint)
+
+                except Exception as ex:
+                    _logger.exception("Error calling listener: %s", ex)
+
+
     def add(self, endpoint):
         """
         Registers an end point and notifies listeners. Does nothing if the
@@ -87,9 +105,14 @@ class ImportsRegistry(object):
         :return: True if the end point has been added
         """
         with self.__lock:
-            # Check if the end point already exist
+            # Check framework UID (avoid to import our own services)
+            if endpoint.framework == self._fw_uid:
+                return False
+
+            # Check if the end point already exists
             if endpoint.uid in self._registry:
                 # Already known end point: do nothing
+                _logger.debug("Already known endpoint")
                 return False
 
             # Store the end point
@@ -105,7 +128,7 @@ class ImportsRegistry(object):
                     listener.endpoint_added(endpoint)
 
                 except Exception as ex:
-                        _logger.exception("Error calling listener: %s", ex)
+                    _logger.exception("Error calling listener: %s", ex)
 
         return True
 
@@ -116,6 +139,7 @@ class ImportsRegistry(object):
 
         :param uid: The UID of the end point
         :param new_properties: The new properties of the end point
+        :return: True if the endpoint is known, else False
         """
         try:
             with self.__lock:
@@ -124,11 +148,11 @@ class ImportsRegistry(object):
 
                 # Replace the stored properties
                 old_properties = stored_endpoint.properties.copy()
-                stored_endpoint.properties = new_properties.copy()
+                stored_endpoint.properties = new_properties
 
         except KeyError:
             # Unknown end point: ignore it
-            return
+            return False
 
         else:
             # Notify listeners
@@ -141,12 +165,15 @@ class ImportsRegistry(object):
                     except Exception as ex:
                         _logger.exception("Error calling listener: %s", ex)
 
+            return True
+
 
     def remove(self, uid):
         """
         Unregisters an end point and notifies listeners
 
         :param uid: The UID of the end point to unregister
+        :return: True if the endpoint was known
         """
         # Remove the end point from the individual storage
         try:
@@ -154,14 +181,20 @@ class ImportsRegistry(object):
 
         except KeyError:
             # Unknown end point
-            return
+            _logger.warning("Unknown end point UID: %s", uid)
+            return False
 
         # Remove it from its framework storage, if any
         try:
-            framework_endpoints = self._frameworks.get(endpoint.framework)
+            framework_endpoints = self._frameworks[endpoint.framework]
             if endpoint in framework_endpoints:
                 framework_endpoints.remove(endpoint)
 
+                if not framework_endpoints:
+                    # Remove framework entry if there is no more endpoint
+                    # from it
+                    del self._frameworks[endpoint.framework]
+
         except (KeyError, ValueError):
             # Ignore the absence of reference in the framework storage
             pass
@@ -175,6 +208,8 @@ class ImportsRegistry(object):
                 except Exception as ex:
                     _logger.exception("Error calling listener: %s", ex)
 
+        return True
+
 
     def lost_framework(self, uid):
         """
@@ -186,6 +221,15 @@ class ImportsRegistry(object):
         endpoints = self._frameworks.pop(uid, None)
         if endpoints:
             for endpoint in endpoints:
+                with self.__lock:
+                    # Remove endpoint from registry
+                    try:
+                        del self._registry[endpoint.uid]
+
+                    except KeyError:
+                        # The endpoint may have been removed by a listener
+                        pass
+
                 # Notify listeners
                 if self._listeners:
                     for listener in self._listeners[:]:
@@ -196,10 +240,21 @@ class ImportsRegistry(object):
                             _logger.exception("Error calling listener: %s", ex)
 
 
+    @Validate
+    def validate(self, context):
+        """
+        Component validated
+        """
+        # Get the framework UID
+        self._fw_uid = context.get_property(pelix.framework.FRAMEWORK_UID)
+
+
     @Invalidate
     def invalidate(self, context):
         """
         Component invalidated: clean up storage
         """
+        # Clean up
+        self._fw_uid = None
         self._frameworks.clear()
         self._registry.clear()
