@@ -4,14 +4,14 @@
 Pelix OSGi-like services packages
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.1
+:version: 0.5.6
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ Pelix OSGi-like services packages
 """
 
 # Module version
-__version_info__ = (0, 1, 0)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -38,6 +38,11 @@ __docformat__ = "restructuredtext en"
 FACTORY_EVENT_ADMIN = "pelix-services-eventadmin-factory"
 """ Name of the EventAdmin component factory """
 
+FACTORY_EVENT_ADMIN_MQTT = "pelix-services-eventadmin-mqtt-factory"
+""" Name of the component factory of the MQTT bridge for EventAdmin """
+
+#-------------------------------------------------------------------------------
+
 SERVICE_EVENT_ADMIN = "pelix.services.eventadmin"
 """ Specification of the EventAdmin service """
 
@@ -56,15 +61,21 @@ EVENT_PROP_FRAMEWORK_UID = "event.sender.framework.uid"
 EVENT_PROP_TIMESTAMP = "event.timestamp"
 """ Time stamp of the event, compute during the call of send() or post() """
 
+EVENT_PROP_PROPAGATE = "event.propagate"
+""" If present in event properties, the event can be propagated through MQTT """
+
 #-------------------------------------------------------------------------------
 
-SERVICE_CONFIGURATION_ADMIN = "pelix.services.configadmin"
+SERVICE_CONFIGURATION_ADMIN = "pelix.configadmin"
 """ Specification of the ConfigurationAdmin service """
 
-SERVICE_CONFIGADMIN_MANAGED = "pelix.services.configadmin.managed"
+SERVICE_CONFIGADMIN_MANAGED = "pelix.configadmin.managed"
 """ Specification of a service managed by ConfigurationAdmin """
 
-SERVICE_CONFIGADMIN_PERSISTENCE = "pelix.services.configadmin.persistence"
+SERVICE_CONFIGADMIN_MANAGED_FACTORY = "pelix.configadmin.managed.factory"
+""" Specification of a factory managed by ConfigurationAdmin """
+
+SERVICE_CONFIGADMIN_PERSISTENCE = "pelix.configadmin.persistence"
 """ Specification of a ConfigurationAdmin storage service """
 
 FACTORY_CONFIGADMIN_JSON = "pelix-configadmin-persistence-json-factory"
@@ -89,3 +100,20 @@ SERVICE_FILEINSTALL_LISTENERS = 'pelix.services.fileinstall.listener'
 
 PROP_FILEINSTALL_FOLDER = 'fileinstall.folder'
 """ Path to the folder to look after, in white board pattern """
+
+#-------------------------------------------------------------------------------
+
+SERVICE_MQTT_CONNECTOR_FACTORY = 'pelix.mqtt.factory'
+""" Specification of an MQTT connection factory """
+
+MQTT_CONNECTOR_FACTORY_PID = 'mqtt.connector'
+""" PID of the MQTT connection factory """
+
+SERVICE_MQTT_CONNECTION = 'pelix.mqtt.connection'
+""" Specification of an MQTT connection service """
+
+SERVICE_MQTT_LISTENER = 'pelix.mqtt.listener'
+""" Specification of an MQTT message listener """
+
+PROP_MQTT_TOPICS = 'pelix.mqtt.topics'
+""" List of the topics a listener wants to subscribes to """
\ No newline at end of file
