@@ -4,14 +4,14 @@
 An EventAdmin-like implementation for Pelix: a publish-subscribe service
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.2
+:version: 0.2.1
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ An EventAdmin-like implementation for Pelix: a publish-subscribe service
 """
 
 # Module version
-__version_info__ = (0, 2, 0)
+__version_info__ = (0, 2, 1)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -38,12 +38,14 @@ __docformat__ = "restructuredtext en"
 # Pelix
 from pelix.ipopo.decorators import ComponentFactory, Provides, Property, \
     Validate, Invalidate
+from pelix.utilities import to_iterable
 import pelix.framework
 import pelix.ldapfilter
 import pelix.services
 import pelix.threadpool
 
 # Standard library
+import copy
 import fnmatch
 import logging
 import time
@@ -101,14 +103,22 @@ class EventAdmin(object):
             # Check the LDAP filter
             ldap_filter = svc_ref.get_property(pelix.services.PROP_EVENT_FILTER)
             if self.__match_filter(properties, ldap_filter):
+                # Get the service ID
+                svc_id = svc_ref.get_property(pelix.framework.SERVICE_ID)
+
                 # Filter matches the event, test the topic
-                topics = svc_ref.get_property(pelix.services.PROP_EVENT_TOPICS)
-                for handled_topic in topics:
-                    if fnmatch.fnmatch(topic, handled_topic):
-                        # Full match, keep the service ID
-                        handlers.append(svc_ref.get_property(\
-                                                 pelix.framework.SERVICE_ID))
-                        break
+                topics = to_iterable(svc_ref.get_property(\
+                                              pelix.services.PROP_EVENT_TOPICS),
+                                     True)
+                if not topics:
+                    # Filter matches, and no topic filter given: notify it
+                    handlers.append(svc_id)
+                else:
+                    for handled_topic in to_iterable(topics, False):
+                        if fnmatch.fnmatch(topic, handled_topic):
+                            # Full match, keep the service ID
+                            handlers.append(svc_id)
+                            break
 
         return handlers
 
@@ -179,7 +189,7 @@ class EventAdmin(object):
                 ref, handler = self.__get_service(handler_id)
                 if handler is not None:
                     # Use a copy of the properties each time
-                    handler.handle_event(topic, properties.copy())
+                    handler.handle_event(topic, copy.deepcopy(properties))
 
             except Exception as ex:
                 _logger.exception("Error notifying event handler %d: %s (%s)",
