@@ -7,14 +7,14 @@ Provides a console interface for the Pelix shell, based on readline when
 available.
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.1.3
+:version: 0.1.4
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -30,7 +30,7 @@ available.
 """
 
 # Module version
-__version_info__ = (0, 1, 3)
+__version_info__ = (0, 1, 4)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -39,7 +39,8 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 # Shell constants
-from pelix.shell import SHELL_SERVICE_SPEC
+from pelix.constants import BundleActivator
+from pelix.shell import SERVICE_SHELL
 import pelix.framework as pelix
 
 # Standard library
@@ -98,7 +99,28 @@ class InteractiveShell(object):
         self.search_shell()
 
         # Register as a service listener
-        self._context.add_service_listener(self, None, SHELL_SERVICE_SPEC)
+        self._context.add_service_listener(self, None, SERVICE_SHELL)
+
+
+    def _readline_prompt(self):
+        """
+        Prompt using the readline module (no pre-flush)
+
+        :return: The command line
+        """
+        sys.stdout.flush()
+        return safe_input(self._shell.get_ps1())
+
+
+    def _normal_prompt(self):
+        """
+        Flushes the prompt before requesting the input
+
+        :return: The command line
+        """
+        sys.stdout.write(self._shell.get_ps1())
+        sys.stdout.flush()
+        return safe_input()
 
 
     def loop_input(self, on_quit=None):
@@ -111,6 +133,10 @@ class InteractiveShell(object):
         try:
             first_prompt = True
 
+            # Set up the prompt
+            prompt = self._readline_prompt if readline is not None \
+                        else self._normal_prompt
+
             while not self._stop_event.is_set():
                 # Wait for the shell to be there
                 # Before Python 2.7, wait() doesn't return a result
@@ -121,12 +147,8 @@ class InteractiveShell(object):
                         sys.stdout.write(self._shell.get_banner())
                         first_prompt = False
 
-                    # Print the prompt
-                    sys.stdout.write(self._shell.get_ps1())
-                    sys.stdout.flush()
-
                     # Read the next line
-                    line = safe_input()
+                    line = prompt()
 
                     with self._lock:
                         if self._shell_event.is_set():
@@ -140,7 +162,9 @@ class InteractiveShell(object):
 
         except (EOFError, KeyboardInterrupt, SystemExit):
             # Input closed or keyboard interruption
-            self._stop_event.set()
+            pass
+
+        self._stop_event.set()
 
         sys.stdout.write('Bye !\n')
         sys.stdout.flush()
@@ -167,13 +191,26 @@ class InteractiveShell(object):
             else:
                 # Complete with name space names or default commands
                 prefix = text
-                possibilities = self._shell.get_commands(None)
-                possibilities.extend(['{0}.'.format(namespace)
-                                      for namespace in self._shell.get_namespaces()])
+
+                # Default commands goes first...
+                possibilities = [command
+                                 for command in self._shell.get_commands(None)
+                                 if command.startswith(prefix)]
+
+                # ... then name spaces
+                namespaces = self._shell.get_namespaces()
+                possibilities.extend('{0}.'.format(namespace)
+                                     for namespace in namespaces
+                                     if namespace.startswith(prefix))
+
+                # ... then commands in those name spaces
+                possibilities.extend('{0}.{1}'.format(namespace, command)
+                            for namespace in namespaces if namespace is not None
+                            for command in self._shell.get_commands(namespace)
+                            if command.startswith(prefix))
 
                 # Filter methods according to the prefix
-                self._readline_matches = [entry for entry in possibilities
-                                          if entry.startswith(prefix)]
+                self._readline_matches = possibilities
 
             if not self._readline_matches:
                 return None
@@ -195,7 +232,7 @@ class InteractiveShell(object):
                 # A shell is already there
                 return
 
-            reference = self._context.get_service_reference(SHELL_SERVICE_SPEC)
+            reference = self._context.get_service_reference(SERVICE_SHELL)
             if reference is not None:
                 self.set_shell(reference)
 
@@ -262,7 +299,6 @@ class InteractiveShell(object):
 
             self._shell_ref = None
             self._shell = None
-            self._readline_matches = None
 
 
     def stop(self):
@@ -284,6 +320,7 @@ class InteractiveShell(object):
 
 # ------------------------------------------------------------------------------
 
+@BundleActivator
 class Activator(object):
     """
     The bundle activator
@@ -363,8 +400,3 @@ if __name__ == '__main__':
 
     # Run the entry point
     main()
-
-else:
-    # Imported as a module:
-    # Creates the Pelix activator
-    activator = Activator()
