@@ -6,14 +6,14 @@ Pelix shell bundle.
 Provides the basic command parsing and execution support to make a Pelix shell.
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.2.3
+:version: 0.2.5
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@ Provides the basic command parsing and execution support to make a Pelix shell.
 """
 
 # Module version
-__version_info__ = (0, 2, 3)
+__version_info__ = (0, 2, 5)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -38,8 +38,8 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 # Shell constants
-from pelix.shell import SHELL_SERVICE_SPEC, SHELL_COMMAND_SPEC, \
-    SHELL_UTILS_SERVICE_SPEC
+from pelix.shell import SERVICE_SHELL, SERVICE_SHELL_COMMAND, \
+    SERVICE_SHELL_UTILS
 
 # Pelix modules
 from pelix.utilities import to_str, to_bytes
@@ -48,6 +48,7 @@ import pelix.framework as pelix
 
 # Standard library
 import inspect
+import linecache
 import logging
 import os
 import shlex
@@ -180,9 +181,11 @@ class IOHandler(object):
         if in_stream is sys.stdin:
             # Warning: conflicts with the console
             self.prompt = safe_input
+        else:
+            self.prompt = self._prompt
 
 
-    def prompt(self, prompt=None):
+    def _prompt(self, prompt=None):
         """
         Reads a line written by the user
 
@@ -227,8 +230,9 @@ class IOHandler(object):
             self.write('\n')
 
         else:
-            # Format the line
-            line = line.format(*args, **kwargs)
+            # Format the line, if arguments have been given
+            if args or kwargs:
+                line = line.format(*args, **kwargs)
 
             # Write it
             self.write(line)
@@ -254,8 +258,9 @@ class IOHandler(object):
             line = ""
 
         else:
-            # Format the line
-            line = line.format(*args, **kwargs)
+            # Format the line, if arguments have been given
+            if args or kwargs:
+                line = line.format(*args, **kwargs)
 
             # Remove the trailing line feed
             if line[-1] == '\n':
@@ -416,6 +421,8 @@ class Shell(object):
         self.register_command(None, "threads", self.threads_list)
         self.register_command(None, "thread", self.thread_details)
 
+        self.register_command(None, "echo", self.echo)
+
         self.register_command(None, "help", self.print_help)
         self.register_command(None, "?", self.print_help)
 
@@ -567,20 +574,52 @@ class Shell(object):
         :param command: A command name
         :return: A list of name spaces
         """
-        if command in self._commands[DEFAULT_NAMESPACE]:
-            # Command is in the default name space, use it directly
-            return [DEFAULT_NAMESPACE]
-
         # Look for the spaces where the command name appears
         namespaces = []
         for namespace, commands in self._commands.items():
             if command in commands:
                 namespaces.append(namespace)
 
+        # Sort name spaces
         namespaces.sort()
+
+        # Default name space must always come first
+        try:
+            namespaces.remove(DEFAULT_NAMESPACE)
+            namespaces.insert(0, DEFAULT_NAMESPACE)
+
+        except ValueError:
+            # Default name space wasn't present
+            pass
+
         return namespaces
 
 
+    def get_ns_commands(self, cmd_name):
+        """
+        Retrieves the possible name spaces and commands associated to the given
+        command name.
+
+        :param cmd_name: The given command name
+        :return: A list of 2-tuples (name space, command)
+        :raise ValueError: Unknown command name
+        """
+        namespace, command = _split_ns_command(cmd_name)
+        if not namespace:
+            # Name space not given, look for the commands
+            spaces = self.__find_command_ns(command)
+            if not spaces:
+                # Unknown command
+                raise ValueError("Unknown command {0}".format(command))
+
+            else:
+                # Return a sorted list of tuples
+                return sorted((namespace, command) for namespace in spaces)
+
+        # Single match
+        return [(namespace, command)]
+
+
     def get_ns_command(self, cmd_name):
         """
         Retrieves the name space and the command associated to the given
@@ -600,8 +639,14 @@ class Shell(object):
 
             elif len(spaces) > 1:
                 # Multiple possibilities
-                raise ValueError("Multiple name spaces for {0}: {1}" \
-                                 .format(command, spaces))
+                if spaces[0] == DEFAULT_NAMESPACE:
+                    # Default name space has priority
+                    namespace = DEFAULT_NAMESPACE
+
+                else:
+                    # Ambiguous name
+                    raise ValueError("Multiple name spaces for {0}: {1}" \
+                                     .format(command, ', '.join(spaces)))
 
             else:
                 # Use the found name space
@@ -622,13 +667,19 @@ class Shell(object):
         # Convert the line into a string
         cmdline = to_str(cmdline)
 
-        line_split = shlex.split(cmdline, True, True)
-        if not line_split:
-            return False
-
         # Prepare the I/O handler
         io_handler = IOHandler(stdin, stdout)
 
+        try:
+            line_split = shlex.split(cmdline, True, True)
+
+        except ValueError as ex:
+            io_handler.write_line("Error reading line: {0}", ex)
+            return False
+
+        if not line_split:
+            return False
+
         try:
             # Extract command information
             namespace, command = self.get_ns_command(line_split[0])
@@ -662,8 +713,9 @@ class Shell(object):
 
         except TypeError as ex:
             # Invalid arguments...
-            _logger.exception("Error calling %s.%s: %s", namespace, command, ex)
+            _logger.error("Error calling %s.%s: %s", namespace, command, ex)
             io_handler.write_line("Invalid method call: {0}", ex)
+            self.__print_namespace_help(io_handler, namespace, command)
             return False
 
         except Exception as ex:
@@ -722,6 +774,13 @@ class Shell(object):
             return []
 
 
+    def echo(self, io_handler, *words):
+        """
+        Echoes the given words
+        """
+        io_handler.write_line(' '.join(words))
+
+
     def bundle_details(self, io_handler, bundle_id):
         """
         Prints the details of the bundle with the given ID or name
@@ -737,12 +796,14 @@ class Shell(object):
             for bundle in self._context.get_bundles():
                 if bundle.get_symbolic_name() == bundle_id:
                     break
+            else:
+                # Bundle not found
+                bundle = None
 
         else:
             # Integer ID: direct access
             try:
                 bundle = self._context.get_bundle(bundle_id)
-
             except constants.BundleException:
                 pass
 
@@ -752,10 +813,10 @@ class Shell(object):
             return
 
         lines = []
-        lines.append("ID      : {0}".format(bundle.get_bundle_id()))
-        lines.append("Name    : {0}".format(bundle.get_symbolic_name()))
-        lines.append("Version : {0}".format(bundle.get_version()))
-        lines.append("State   : {0}".format(self._utils.bundlestate_to_str(
+        lines.append("ID......: {0}".format(bundle.get_bundle_id()))
+        lines.append("Name....: {0}".format(bundle.get_symbolic_name()))
+        lines.append("Version.: {0}".format(bundle.get_version()))
+        lines.append("State...: {0}".format(self._utils.bundlestate_to_str(
                                                         bundle.get_state())))
         lines.append("Location: {0}".format(bundle.get_location()))
         lines.append("Published services:")
@@ -764,7 +825,6 @@ class Shell(object):
             if services:
                 for svc_ref in services:
                     lines.append("\t{0}".format(svc_ref))
-
             else:
                 lines.append("\tn/a")
 
@@ -790,9 +850,10 @@ class Shell(object):
         io_handler.write('\n'.join(lines))
 
 
-    def bundles_list(self, io_handler):
+    def bundles_list(self, io_handler, name=None):
         """
-        Lists the bundles in the framework and their state
+        Lists the bundles in the framework and their state. Possibility to
+        filter on the bundle name.
         """
         # Head of the table
         headers = ('ID', 'Name', 'State', 'Version')
@@ -803,6 +864,11 @@ class Shell(object):
         # The framework is not in the result of get_bundles()
         bundles.insert(0, self._context.get_bundle(0))
 
+        if name is not None:
+            # Filter the list
+            bundles = [bundle for bundle in bundles
+                       if name in bundle.get_symbolic_name()]
+
         # Make the entries
         lines = [[str(entry) for entry in
                         (bundle.get_bundle_id(),
@@ -813,7 +879,12 @@ class Shell(object):
 
         # Print'em all
         io_handler.write(self._utils.make_table(headers, lines))
-        io_handler.write_line("{0} bundles installed", len(lines))
+
+        if name is None:
+            io_handler.write_line("{0} bundles installed", len(lines))
+
+        else:
+            io_handler.write_line("{0} filtered bundles", len(lines))
 
 
     def service_details(self, io_handler, service_id):
@@ -828,15 +899,15 @@ class Shell(object):
             return
 
         lines = []
-        lines.append("ID    : {0}".format(svc_ref.get_property(
+        lines.append("ID............: {0}".format(svc_ref.get_property(
                                             constants.SERVICE_ID)))
-        lines.append("Rank  : {0}".format(svc_ref.get_property(
+        lines.append("Rank..........: {0}".format(svc_ref.get_property(
                                             constants.SERVICE_RANKING)))
-        lines.append("Specs : {0}".format(svc_ref.get_property(
+        lines.append("Specifications: {0}".format(svc_ref.get_property(
                                             constants.OBJECTCLASS)))
-        lines.append("Bundle: {0}".format(svc_ref.get_bundle()))
-        lines.append("Properties:")
-        for key, value in svc_ref.get_properties().items():
+        lines.append("Bundle........: {0}".format(svc_ref.get_bundle()))
+        lines.append("Properties....:")
+        for key, value in sorted(svc_ref.get_properties().items()):
             lines.append("\t{0} = {1}".format(key, value))
 
         lines.append("Bundles using this service:")
@@ -847,9 +918,10 @@ class Shell(object):
         io_handler.write('\n'.join(lines))
 
 
-    def services_list(self, io_handler):
+    def services_list(self, io_handler, specification=None):
         """
-        Lists the services in the framework
+        Lists the services in the framework. Possibility to filter on an exact
+        specification.
         """
         # Head of the table
         headers = ('ID', 'Specifications', 'Bundle', 'Ranking')
@@ -860,6 +932,11 @@ class Shell(object):
         # Use the reverse order (ascending service IDs instead of descending)
         references.reverse()
 
+        if specification is not None:
+            # Filter on specifications
+            references = [ref for ref in references if specification
+                          in ref.get_property(constants.OBJECTCLASS)]
+
         # Construct the list of services
         lines = [[str(entry) for entry in
                         (ref.get_property(constants.SERVICE_ID),
@@ -868,9 +945,13 @@ class Shell(object):
                          ref.get_property(constants.SERVICE_RANKING))]
                  for ref in references]
 
-        # Print'em all
-        io_handler.write(self._utils.make_table(headers, lines))
-        io_handler.write_line("{0} services registered", len(lines))
+        if not lines and specification:
+            # No matching service found
+            io_handler.write_line("No service provides '{0}'", specification)
+        else:
+            # Print'em all
+            io_handler.write(self._utils.make_table(headers, lines))
+            io_handler.write_line("{0} services registered", len(lines))
 
 
     def __extract_help(self, method):
@@ -940,6 +1021,35 @@ class Shell(object):
         io_handler.write_line("\t\t{0}", doc)
 
 
+    def __print_namespace_help(self, io_handler, namespace, cmd_name=None):
+        """
+        Prints the documentation of all the commands in the given name space,
+        or only of the given command
+
+        :param io_handler: I/O Handler
+        :param namespace: Name space of the command
+        :param cmd_name: Name of the command to show, None to show them all
+        """
+        io_handler.write_line("=== Name space '{0}' ===", namespace)
+
+        # Get all commands in this name space
+        if cmd_name is None:
+            names = [command for command in self._commands[namespace]]
+            names.sort()
+
+        else:
+            names = [cmd_name]
+
+        first_cmd = True
+        for command in names:
+            if not first_cmd:
+                # Print an empty line
+                io_handler.write_line('\n')
+
+            self.__print_command_help(io_handler, namespace, command)
+            first_cmd = False
+
+
     def print_help(self, io_handler, command=None):
         """
         Prints the available methods and their documentation, or the
@@ -947,19 +1057,33 @@ class Shell(object):
         """
         if command:
             # Single command mode
+            if command in self._commands:
+                # Argument is a name space
+                self.__print_namespace_help(io_handler, command)
+                was_namespace = True
+            else:
+                was_namespace = False
+
+            # Also print the name of matching commands
             try:
                 # Extract command name space and name
-                namespace, cmd_name = self.get_ns_command(command)
+                possibilities = self.get_ns_commands(command)
 
             except ValueError as ex:
                 # Unknown command
-                io_handler.write_line(str(ex))
-                return False
+                if not was_namespace:
+                    # ... and no name space were matching either -> error
+                    io_handler.write_line(str(ex))
+                    return False
 
             else:
-                # Print the help
-                io_handler.write_line("* Name space '{0}':", namespace)
-                self.__print_command_help(io_handler, namespace, cmd_name)
+                # Print the help of the found command
+                if was_namespace:
+                    # Give some space
+                    io_handler.write_line('\n\n')
+
+                for namespace, cmd_name in possibilities:
+                    self.__print_namespace_help(io_handler, namespace, cmd_name)
 
         else:
             # Get all name spaces
@@ -968,15 +1092,15 @@ class Shell(object):
             namespaces.sort()
             namespaces.insert(0, DEFAULT_NAMESPACE)
 
+            first_ns = True
             for namespace in namespaces:
-                io_handler.write_line("* Name space '{0}':", namespace)
+                if not first_ns:
+                    # Add empty lines
+                    io_handler.write_line('\n\n')
 
-                # Get all commands in this name space
-                names = [command for command in self._commands[namespace]]
-                names.sort()
-
-                for cmd_name in names:
-                    self.__print_command_help(io_handler, namespace, cmd_name)
+                # Print the help of all commands
+                self.__print_namespace_help(io_handler, namespace)
+                first_ns = False
 
 
     def properties_list(self, io_handler):
@@ -1004,7 +1128,12 @@ class Shell(object):
         Prints the value of the given property, looking into
         framework properties then environment variables.
         """
-        io_handler.write_line(str(self._context.get_property(name)))
+        value = self._context.get_property(name)
+        if value is None:
+            # Avoid printing "None"
+            value = ""
+
+        io_handler.write_line(str(value))
 
 
     def environment_list(self, io_handler):
@@ -1096,54 +1225,184 @@ class Shell(object):
             lines = []
             lines.append('Thread ID: {0} - Name: {1}'.format(thread_id, name))
             lines.append('Stack trace:')
-            lines.extend((line.rstrip()
-                          for line in traceback.format_stack(stack)))
 
-        lines.append("")
-        io_handler.write('\n'.join(lines))
+            trace_lines = []
+            frame = stack
+            while frame is not None:
+                # Store the line information
+                trace_lines.append(self.__format_frame_info(frame))
+
+                # Previous frame...
+                frame = frame.f_back
+
+            # Reverse the lines
+            trace_lines.reverse()
+
+            # Add them to the printed lines
+            lines.extend(trace_lines)
+
+            lines.append('')
+            io_handler.write('\n'.join(lines))
+
+
+    def __format_frame_info(self, frame):
+        """
+        Formats the given stack frame to show its position in the code and
+        part of its context
+
+        :param frame: A stack frame
+        """
+        # Same as in traceback.extract_stack
+        lineno = frame.f_lineno
+        code = frame.f_code
+        filename = code.co_filename
+        method_name = code.co_name
+        linecache.checkcache(filename)
+
+        output_lines = []
+
+        try:
+            # Try to get the type of the calling object
+            instance = frame.f_locals['self']
+            method_name = '{0}::{1}'.format(type(instance).__name__,
+                                            method_name)
+        except KeyError:
+            # Not called from a bound method
+            pass
+
+        # File & line
+        output_lines.append('  File "{0}", line {1}, in {2}'\
+                            .format(filename, lineno, method_name))
+
+        # Arguments
+        arginfo = inspect.getargvalues(frame)
+        for name in arginfo.args:
+            output_lines.append('    - {0:s} = {1}'\
+                                 .format(name, repr(frame.f_locals[name])))
+
+        if arginfo.varargs:
+            output_lines.append('    - *{0:s} = {1}'\
+                                .format(arginfo.varargs,
+                                        frame.f_locals[arginfo.varargs]))
+
+        if arginfo.keywords:
+            output_lines.append('    - **{0:s} = {1}'\
+                                .format(arginfo.keywords,
+                                        frame.f_locals[arginfo.keywords]))
+
+        # Line block
+        lines = self.__extract_lines(filename, frame.f_globals, lineno, 3)
+        if lines:
+            output_lines.append('')
+            prefix = '      '
+            output_lines.append('{0}{1}'.format(prefix,
+                                            '\n{0}'.format(prefix).join(lines)))
+
+        return '\n'.join(output_lines)
+
+
+    def __extract_lines(self, filename, f_globals, lineno, around):
+        """
+        Extracts a block of lines from the given file
+
+        :param filename: Name of the source file
+        :param f_globals: Globals of the frame of the current code
+        :param lineno: Current line of code
+        :param around: Number of line to print before and after the current one
+        """
+        current_line = linecache.getline(filename, lineno, f_globals)
+        if not current_line:
+            # No data on this line
+            return ''
+
+        lines = []
+        # Add some lines before
+        for pre_lineno in range(lineno - around, lineno):
+            pre_line = linecache.getline(filename, pre_lineno, f_globals)
+            lines.append('{0}'.format(pre_line.rstrip()))
+
+        # The line itself
+        lines.append('{0}'.format(current_line.rstrip()))
+
+        # Add some lines after
+        for pre_lineno in range(lineno + 1, lineno + around + 1):
+            pre_line = linecache.getline(filename, pre_lineno, f_globals)
+            lines.append('{0}'.format(pre_line.rstrip()))
+
+        # Smart left strip
+        minimal_tab = None
+        for line in lines:
+            if line.strip():
+                tab = len(line) - len(line.lstrip())
+                if minimal_tab is None or tab < minimal_tab:
+                    minimal_tab = tab
+
+        if minimal_tab > 0:
+            lines = [line[minimal_tab:] for line in lines]
+
+        # Add some place for a marker
+        marked_line = '>> {0}'.format(lines[around])
+        lines = ['   {0}'.format(line) for line in lines]
+        lines[around] = marked_line
+        lines.append('')
+
+        # Return the lines
+        return lines
 
 
     def quit(self, io_handler):
         """
         Stops the current shell session (raises a KeyboardInterrupt exception)
         """
+        io_handler.write_line("Raising KeyboardInterrupt to stop main thread")
         raise KeyboardInterrupt()
 
 
+    def __get_bundle(self, io_handler, bundle_id):
+        """
+        Retrieves the Bundle object with the given bundle ID. Writes errors
+        through the I/O handler if any.
+
+        :param io_handler: I/O Handler
+        :param bundle_id: String or integer bundle ID
+        :return: The Bundle object matching the given ID, None if not found
+        """
+        try:
+            bundle_id = int(bundle_id)
+            return self._context.get_bundle(bundle_id)
+
+        except (TypeError, ValueError):
+            io_handler.write_line("Invalid bundle ID: {0}", bundle_id)
+
+        except constants.BundleException:
+            io_handler.write_line("Unknown bundle: {0}", bundle_id)
+
+
     def start(self, io_handler, bundle_id):
         """
         Starts the bundle with the given ID
         """
-        bundle_id = int(bundle_id)
-        bundle = self._context.get_bundle(bundle_id)
-        if bundle is None:
-            io_handler.write_line("Unknown bundle: {0}", bundle_id)
-
-        bundle.start()
+        bundle = self.__get_bundle(io_handler, bundle_id)
+        if bundle is not None:
+            bundle.start()
 
 
     def stop(self, io_handler, bundle_id):
         """
         Stops the bundle with the given ID
         """
-        bundle_id = int(bundle_id)
-        bundle = self._context.get_bundle(bundle_id)
-        if bundle is None:
-            io_handler.write_line("Unknown bundle: {0}", bundle_id)
-
-        bundle.stop()
+        bundle = self.__get_bundle(io_handler, bundle_id)
+        if bundle is not None:
+            bundle.stop()
 
 
     def update(self, io_handler, bundle_id):
         """
         Updates the bundle with the given ID
         """
-        bundle_id = int(bundle_id)
-        bundle = self._context.get_bundle(bundle_id)
-        if bundle is None:
-            io_handler.write_line("Unknown bundle: {0}", bundle_id)
-
-        bundle.update()
+        bundle = self.__get_bundle(io_handler, bundle_id)
+        if bundle is not None:
+            bundle.update()
 
 
     def install(self, io_handler, module_name):
@@ -1158,16 +1417,13 @@ class Shell(object):
         """
         Uninstalls the bundle with the given ID
         """
-        bundle_id = int(bundle_id)
-        bundle = self._context.get_bundle(bundle_id)
-        if bundle is None:
-            io_handler.write_line("Unknown bundle: {0}", bundle_id)
-
-        bundle.uninstall()
-
+        bundle = self.__get_bundle(io_handler, bundle_id)
+        if bundle is not None:
+            bundle.uninstall()
 
 # ------------------------------------------------------------------------------
 
+@constants.BundleActivator
 class PelixActivator(object):
     """
     Activator class for Pelix
@@ -1209,17 +1465,17 @@ class PelixActivator(object):
             utils = ShellUtils()
             self._shell = Shell(context, utils)
 
-            self._shell_reg = context.register_service(SHELL_SERVICE_SPEC,
+            self._shell_reg = context.register_service(SERVICE_SHELL,
                                                        self._shell, {})
 
-            self._utils_reg = context.register_service(SHELL_UTILS_SERVICE_SPEC,
+            self._utils_reg = context.register_service(SERVICE_SHELL_UTILS,
                                                        utils, {})
 
             # Register the service listener
-            context.add_service_listener(self, None, SHELL_COMMAND_SPEC)
+            context.add_service_listener(self, None, SERVICE_SHELL_COMMAND)
 
             # Register existing command services
-            refs = context.get_all_service_references(SHELL_COMMAND_SPEC,
+            refs = context.get_all_service_references(SERVICE_SHELL_COMMAND,
                                                       None)
             if refs is not None:
                 for ref in refs:
@@ -1251,7 +1507,3 @@ class PelixActivator(object):
 
         self._shell = None
         _logger.info("Shell services unregistered")
-
-
-# Activator for Pelix
-activator = PelixActivator()
