@@ -6,14 +6,14 @@ iPOPO shell commands
 Provides commands to the Pelix shell to get the state of iPOPO instances.
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.2.2
+:version: 0.2.3
 :status: Beta
 
 ..
 
-    Copyright 2013 isandlaTech
+    Copyright 2014 isandlaTech
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@ Provides commands to the Pelix shell to get the state of iPOPO instances.
 """
 
 # Module version
-__version_info__ = (0, 2, 2)
+__version_info__ = (0, 2, 3)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -37,15 +37,13 @@ __docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
+# Pelix
 from pelix.ipopo.decorators import ComponentFactory, Requires, Provides, \
     Instantiate
+import pelix.ipopo.constants
+import pelix.shell
 
-# iPOPO constants
-from pelix.ipopo.constants import IPOPO_SERVICE_SPECIFICATION
-
-# Shell constants
-from pelix.shell import SHELL_COMMAND_SPEC, SHELL_UTILS_SERVICE_SPEC
-
+# Standard library
 import logging
 
 # ------------------------------------------------------------------------------
@@ -72,9 +70,9 @@ def ipopo_state_to_str(state):
 # ------------------------------------------------------------------------------
 
 @ComponentFactory("ipopo-shell-commands-factory")
-@Requires("_ipopo", IPOPO_SERVICE_SPECIFICATION)
-@Requires("_utils", SHELL_UTILS_SERVICE_SPEC)
-@Provides(SHELL_COMMAND_SPEC)
+@Requires("_ipopo", pelix.ipopo.constants.SERVICE_IPOPO)
+@Requires("_utils", pelix.shell.SERVICE_SHELL_UTILS)
+@Provides(pelix.shell.SERVICE_SHELL_COMMAND)
 @Instantiate("ipopo-shell-commands")
 class IPopoCommands(object):
     """
@@ -102,33 +100,79 @@ class IPopoCommands(object):
         return [("factories", self.list_factories),
                 ("factory", self.factory_details),
                 ("instances", self.list_instances),
+                ("waiting", self.list_waitings),
                 ("instance", self.instance_details),
                 ("instantiate", self.instantiate),
                 ("kill", self.kill),
                 ]
 
 
-    def list_factories(self, io_handler):
+    def list_factories(self, io_handler, name=None):
         """
         Lists the available iPOPO component factories
         """
         header = ('Factory', 'Bundle')
-        lines = [(name, self._ipopo.get_factory_bundle(name))
-                 for name in self._ipopo.get_factories()]
+
+        factories = self._ipopo.get_factories()
+        if name is not None:
+            # Filter factories by name
+            factories = [factory for factory in factories if name in factory]
+
+        lines = sorted((name, self._ipopo.get_factory_bundle(name))
+                       for name in factories)
+
         io_handler.write(self._utils.make_table(header, lines))
-        io_handler.write_line("{0} factories available", len(lines))
+        if name is None:
+            io_handler.write_line("{0} factories available", len(lines))
+        else:
+            io_handler.write_line("{0} filtered factories", len(lines))
 
 
-    def list_instances(self, io_handler):
+    def list_instances(self, io_handler, name=None):
         """
         Lists the active iPOPO component instances
         """
         headers = ('Name', 'Factory', 'State')
-        lines = [(name, factory, ipopo_state_to_str(state))
-                 for name, factory, state in self._ipopo.get_instances()]
+
+        instances = self._ipopo.get_instances()
+        if name is not None:
+            # Filter instances by name
+            instances = [instance for instance in instances
+                         if name in instance[0]]
+
+        # Lines are already sorted
+        lines = ((name, factory, ipopo_state_to_str(state))
+                 for name, factory, state in instances)
 
         io_handler.write(self._utils.make_table(headers, lines))
-        io_handler.write_line("{0} components instantiated", len(lines))
+        if name is None:
+            io_handler.write_line("{0} components running", len(instances))
+        else:
+            io_handler.write_line("{0} filtered components", len(instances))
+
+
+    def list_waitings(self, io_handler, name=None):
+        """
+        Lists the components waiting to be instantiated
+        """
+        headers = ('Name', 'Factory', 'Missing handlers')
+
+        components = self._ipopo.get_waiting_components()
+        if name is not None:
+            # Filter components by name
+            components = [component for component in components
+                         if name in component[0]]
+
+        # Lines are already sorted
+        lines = ((name, factory, ', '.join(missing))
+                 for name, factory, missing in components)
+
+        io_handler.write(self._utils.make_table(headers, lines))
+        if name is None:
+            io_handler.write_line("{0} components in the waiting queue",
+                                  len(components))
+        else:
+            io_handler.write_line("{0} filtered components", len(components))
 
 
     def factory_details(self, io_handler, name):
@@ -198,14 +242,14 @@ class IPopoCommands(object):
 
         # Basic information
         lines = []
-        lines.append("Name     : {0}".format(details["name"]))
-        lines.append("Factory  : {0}".format(details["factory"]))
+        lines.append("Name.....: {0}".format(details["name"]))
+        lines.append("Factory..: {0}".format(details["factory"]))
         lines.append("Bundle ID: {0}".format(details["bundle_id"]))
-        lines.append("State    : {0}".format(
+        lines.append("State....: {0}".format(
                                          ipopo_state_to_str(details["state"])))
 
         # Provided services
-        lines.append("Services :")
+        lines.append("Services.:")
         lines.extend("\t{0}".format(svc_reference)
                      for svc_reference in details["services"].values())
 
@@ -216,20 +260,19 @@ class IPopoCommands(object):
             lines.append("\t\tSpecification: {0}"\
                          .format(infos['specification']))
             if "filter" in infos:
-                lines.append("\t\tFilter   : {0}".format(infos["filter"]))
-            lines.append("\t\tOptional : {0}".format(infos["optional"]))
-            lines.append("\t\tAggregate: {0}".format(infos["aggregate"]))
+                lines.append("\t\tFilter......: {0}".format(infos["filter"]))
+            lines.append("\t\tOptional.....: {0}".format(infos["optional"]))
+            lines.append("\t\tAggregate....: {0}".format(infos["aggregate"]))
 
-            lines.append("\t\tHandler  : {0}".format(infos["handler"]))
-            lines.append("\t\tBindings :")
+            lines.append("\t\tHandler......: {0}".format(infos["handler"]))
+            lines.append("\t\tBindings:")
             for ref in infos["bindings"]:
                 lines.append('\t\t\t{0}'.format(ref))
 
         # Properties
         lines.append("Properties:")
         lines.append(self._utils.make_table(("Key", "Value"),
-                                            details['properties'].items(),
-                                            "\t"))
+                                sorted(details['properties'].items()), "\t"))
 
         lines.append("")
         io_handler.write('\n'.join(lines))
@@ -252,6 +295,7 @@ class IPopoCommands(object):
 
         except Exception as ex:
             io_handler.write_line("Error instantiating the component: {0}", ex)
+            _logger.exception("Error instantiating the component")
 
 
     def kill(self, io_handler, name):
