@@ -4,9 +4,9 @@
 Utility methods and decorators
 
 :author: Thomas Calmant
-:copyright: Copyright 2013, isandlaTech
+:copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.5
+:version: 0.5.6
 :status: Beta
 
 ..
@@ -42,7 +42,7 @@ import traceback
 # ------------------------------------------------------------------------------
 
 # Module version
-__version_info__ = (0, 5, 5)
+__version_info__ = (0, 5, 6)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -418,3 +418,30 @@ else:
             return data
 
         return data.decode(encoding)
+
+# ------------------------------------------------------------------------------
+
+def to_iterable(value, allow_none=True):
+    """
+    Tries to convert the given value to an iterable, if necessary.
+    If the given value is a list, a list is returned; if it is a string, a list
+    containing one string is returned, ...
+
+    :param value: Any object
+    :param allow_none: If True, the method returns None if value is None, else
+                       it returns an empty list
+    :return: A list containing the given string, or the given value
+    """
+    if value is None:
+        # None given
+        if allow_none:
+            return None
+        else:
+            return []
+
+    elif isinstance(value, (list, tuple, set, frozenset)):
+        # Iterable given, return it as-is
+        return value
+
+    # Return a one-value list
+    return [value]
